@@ -25,7 +25,6 @@
 #include "dxva2api.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(evr);
@@ -148,7 +147,7 @@ static ULONG WINAPI tracked_async_result_AddRef(IMFAsyncResult *iface)
     struct tracked_async_result *result = impl_from_IMFAsyncResult(iface);
     ULONG refcount = InterlockedIncrement(&result->refcount);
 
-    TRACE("%p, %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -158,7 +157,7 @@ static ULONG WINAPI tracked_async_result_Release(IMFAsyncResult *iface)
     struct tracked_async_result *result = impl_from_IMFAsyncResult(iface);
     ULONG refcount = InterlockedDecrement(&result->refcount);
 
-    TRACE("%p, %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -168,7 +167,7 @@ static ULONG WINAPI tracked_async_result_Release(IMFAsyncResult *iface)
             IUnknown_Release(result->object);
         if (result->state)
             IUnknown_Release(result->state);
-        heap_free(result);
+        free(result);
     }
 
     return refcount;
@@ -202,7 +201,7 @@ static HRESULT WINAPI tracked_async_result_SetStatus(IMFAsyncResult *iface, HRES
 {
     struct tracked_async_result *result = impl_from_IMFAsyncResult(iface);
 
-    TRACE("%p, %#x.\n", iface, status);
+    TRACE("%p, %#lx.\n", iface, status);
 
     result->result.hrStatusResult = status;
 
@@ -250,7 +249,7 @@ static HRESULT create_async_result(IUnknown *object, IMFAsyncCallback *callback,
 {
     struct tracked_async_result *result;
 
-    result = heap_alloc_zero(sizeof(*result));
+    result = calloc(1, sizeof(*result));
     if (!result)
         return E_OUTOFMEMORY;
 
@@ -352,7 +351,7 @@ static void video_sample_create_tracking_thread(void)
         WaitForSingleObject(ready_event, INFINITE);
         CloseHandle(ready_event);
 
-        TRACE("Create tracking thread %#x.\n", tracking_thread.tid);
+        TRACE("Create tracking thread %#lx.\n", tracking_thread.tid);
     }
 
     LeaveCriticalSection(&tracking_thread_cs);
@@ -451,7 +450,7 @@ static ULONG WINAPI sample_allocator_AddRef(IMFVideoSampleAllocator *iface)
     struct sample_allocator *allocator = impl_from_IMFVideoSampleAllocator(iface);
     ULONG refcount = InterlockedIncrement(&allocator->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -464,13 +463,13 @@ static void sample_allocator_release_samples(struct sample_allocator *allocator)
     {
         list_remove(&iter->entry);
         IMFSample_Release(iter->sample);
-        heap_free(iter);
+        free(iter);
     }
 
     LIST_FOR_EACH_ENTRY_SAFE(iter, iter2, &allocator->used_samples, struct queued_sample, entry)
     {
         list_remove(&iter->entry);
-        heap_free(iter);
+        free(iter);
     }
 }
 
@@ -479,7 +478,7 @@ static ULONG WINAPI sample_allocator_Release(IMFVideoSampleAllocator *iface)
     struct sample_allocator *allocator = impl_from_IMFVideoSampleAllocator(iface);
     ULONG refcount = InterlockedDecrement(&allocator->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -489,7 +488,7 @@ static ULONG WINAPI sample_allocator_Release(IMFVideoSampleAllocator *iface)
             IDirect3DDeviceManager9_Release(allocator->device_manager);
         sample_allocator_release_samples(allocator);
         DeleteCriticalSection(&allocator->cs);
-        heap_free(allocator);
+        free(allocator);
     }
 
     return refcount;
@@ -576,7 +575,7 @@ static HRESULT sample_allocator_create_samples(struct sample_allocator *allocato
 
         if (FAILED(hr))
         {
-            WARN("Failed to get processor service, %#x.\n", hr);
+            WARN("Failed to get processor service, %#lx.\n", hr);
             return hr;
         }
     }
@@ -618,7 +617,7 @@ static HRESULT sample_allocator_create_samples(struct sample_allocator *allocato
             break;
         }
 
-        queued_sample = heap_alloc(sizeof(*queued_sample));
+        queued_sample = malloc(sizeof(*queued_sample));
         queued_sample->sample = sample;
         list_add_tail(&allocator->free_samples, &queued_sample->entry);
         allocator->free_sample_count++;
@@ -639,7 +638,7 @@ static HRESULT WINAPI sample_allocator_InitializeSampleAllocator(IMFVideoSampleA
     struct sample_allocator *allocator = impl_from_IMFVideoSampleAllocator(iface);
     HRESULT hr;
 
-    TRACE("%p, %u, %p.\n", iface, sample_count, media_type);
+    TRACE("%p, %lu, %p.\n", iface, sample_count, media_type);
 
     if (!sample_count)
         sample_count = 1;
@@ -860,7 +859,7 @@ HRESULT WINAPI MFCreateVideoSampleAllocator(REFIID riid, void **obj)
 
     TRACE("%s, %p.\n", debugstr_guid(riid), obj);
 
-    if (!(object = heap_alloc_zero(sizeof(*object))))
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->IMFVideoSampleAllocator_iface.lpVtbl = &sample_allocator_vtbl;
@@ -913,7 +912,7 @@ static ULONG WINAPI video_sample_AddRef(IMFSample *iface)
     struct video_sample *sample = impl_from_IMFSample(iface);
     ULONG refcount = InterlockedIncrement(&sample->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -921,21 +920,23 @@ static ULONG WINAPI video_sample_AddRef(IMFSample *iface)
 static ULONG WINAPI video_sample_Release(IMFSample *iface)
 {
     struct video_sample *sample = impl_from_IMFSample(iface);
-    ULONG refcount;
+    ULONG refcount = InterlockedDecrement(&sample->refcount);
+    IMFAsyncResult *tracked_result = NULL;
 
-    IMFSample_LockStore(sample->sample);
-    if (sample->tracked_result && sample->tracked_refcount == (sample->refcount - 1))
+    EnterCriticalSection(&sample->cs);
+    if (sample->tracked_result && sample->tracked_refcount == refcount)
     {
-        video_sample_tracking_thread_invoke(sample->tracked_result);
-        IMFAsyncResult_Release(sample->tracked_result);
+        tracked_result = sample->tracked_result;
+        video_sample_tracking_thread_invoke(tracked_result);
         sample->tracked_result = NULL;
         sample->tracked_refcount = 0;
     }
-    IMFSample_UnlockStore(sample->sample);
+    LeaveCriticalSection(&sample->cs);
 
-    refcount = InterlockedDecrement(&sample->refcount);
+    if (tracked_result)
+        IMFAsyncResult_Release(tracked_result);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
@@ -943,7 +944,7 @@ static ULONG WINAPI video_sample_Release(IMFSample *iface)
         if (sample->sample)
             IMFSample_Release(sample->sample);
         DeleteCriticalSection(&sample->cs);
-        heap_free(sample);
+        free(sample);
     }
 
     return refcount;
@@ -1233,7 +1234,7 @@ static HRESULT WINAPI video_sample_SetSampleFlags(IMFSample *iface, DWORD flags)
 {
     struct video_sample *sample = impl_from_IMFSample(iface);
 
-    TRACE("%p, %#x.\n", iface, flags);
+    TRACE("%p, %#lx.\n", iface, flags);
 
     return IMFSample_SetSampleFlags(sample->sample, flags);
 }
@@ -1313,7 +1314,7 @@ static HRESULT WINAPI video_sample_GetBufferByIndex(IMFSample *iface, DWORD inde
 {
     struct video_sample *sample = impl_from_IMFSample(iface);
 
-    TRACE("%p, %u, %p.\n", iface, index, buffer);
+    TRACE("%p, %lu, %p.\n", iface, index, buffer);
 
     return IMFSample_GetBufferByIndex(sample->sample, index, buffer);
 }
@@ -1338,7 +1339,7 @@ static HRESULT WINAPI video_sample_RemoveBufferByIndex(IMFSample *iface, DWORD i
 {
     struct video_sample *sample = impl_from_IMFSample(iface);
 
-    TRACE("%p, %u.\n", iface, index);
+    TRACE("%p, %lu.\n", iface, index);
 
     return IMFSample_RemoveBufferByIndex(sample->sample, index);
 }
@@ -1445,8 +1446,7 @@ static HRESULT WINAPI tracked_video_sample_SetAllocator(IMFTrackedSample *iface,
 
     TRACE("%p, %p, %p.\n", iface, sample_allocator, state);
 
-    IMFSample_LockStore(sample->sample);
-
+    EnterCriticalSection(&sample->cs);
     if (sample->tracked_result)
         hr = MF_E_NOTACCEPTING;
     else
@@ -1463,8 +1463,7 @@ static HRESULT WINAPI tracked_video_sample_SetAllocator(IMFTrackedSample *iface,
             }
         }
     }
-
-    IMFSample_UnlockStore(sample->sample);
+    LeaveCriticalSection(&sample->cs);
 
     return hr;
 }
@@ -1586,7 +1585,7 @@ static ULONG WINAPI surface_buffer_AddRef(IMFMediaBuffer *iface)
     struct surface_buffer *buffer = impl_from_IMFMediaBuffer(iface);
     ULONG refcount = InterlockedIncrement(&buffer->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     return refcount;
 }
@@ -1596,12 +1595,12 @@ static ULONG WINAPI surface_buffer_Release(IMFMediaBuffer *iface)
     struct surface_buffer *buffer = impl_from_IMFMediaBuffer(iface);
     ULONG refcount = InterlockedDecrement(&buffer->refcount);
 
-    TRACE("%p, refcount %u.\n", iface, refcount);
+    TRACE("%p, refcount %lu.\n", iface, refcount);
 
     if (!refcount)
     {
         IUnknown_Release(buffer->surface);
-        heap_free(buffer);
+        free(buffer);
     }
 
     return refcount;
@@ -1704,7 +1703,7 @@ static HRESULT create_surface_buffer(IUnknown *surface, IMFMediaBuffer **buffer)
 {
     struct surface_buffer *object;
 
-    if (!(object = heap_alloc_zero(sizeof(*object))))
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->IMFMediaBuffer_iface.lpVtbl = &surface_buffer_vtbl;
@@ -1726,7 +1725,7 @@ HRESULT WINAPI MFCreateVideoSampleFromSurface(IUnknown *surface, IMFSample **sam
 
     TRACE("%p, %p.\n", surface, sample);
 
-    if (!(object = heap_alloc_zero(sizeof(*object))))
+    if (!(object = calloc(1, sizeof(*object))))
         return E_OUTOFMEMORY;
 
     object->IMFSample_iface.lpVtbl = &video_sample_vtbl;
@@ -1737,7 +1736,7 @@ HRESULT WINAPI MFCreateVideoSampleFromSurface(IUnknown *surface, IMFSample **sam
 
     if (FAILED(hr = MFCreateSample(&object->sample)))
     {
-        heap_free(object);
+        free(object);
         return hr;
     }
 
