@@ -196,7 +196,8 @@ static void * const syscalls[] =
     NtGetNlsSectionPtr,
     NtGetWriteWatch,
     NtImpersonateAnonymousToken,
-    NtInitiatePowerAction ,
+    NtInitializeNlsFiles,
+    NtInitiatePowerAction,
     NtIsProcessInJob,
     NtListenPort,
     NtLoadDriver,
@@ -206,6 +207,7 @@ static void * const syscalls[] =
     NtLockVirtualMemory,
     NtMakeTemporaryObject,
     NtMapViewOfSection,
+    NtMapViewOfSectionEx,
     NtNotifyChangeDirectoryFile,
     NtNotifyChangeKey,
     NtNotifyChangeMultipleKeys,
@@ -334,6 +336,7 @@ static void * const syscalls[] =
     NtUnlockFile,
     NtUnlockVirtualMemory,
     NtUnmapViewOfSection,
+    NtUnmapViewOfSectionEx,
     NtWaitForAlertByThreadId,
     NtWaitForDebugEvent,
     NtWaitForKeyedEvent,
@@ -2032,11 +2035,7 @@ static void load_apiset_dll(void)
     }
     if (!status)
     {
-        ptr = NULL;
-        size = 0;
-        status = NtMapViewOfSection( mapping, NtCurrentProcess(), &ptr,
-                                     is_win64 && wow_peb ? 0x7fffffff : 0, 0, NULL,
-                                     &size, ViewShare, 0, PAGE_READONLY );
+        status = map_section( mapping, &ptr, &size, PAGE_READONLY );
         NtClose( mapping );
     }
     if (!status)
