@@ -74,89 +74,6 @@ static void DEFWND_HandleWindowPosChanged( HWND hwnd, const WINDOWPOS *winpos )
 }
 
 
-/***********************************************************************
- *           DEFWND_SetTextA
- *
- * Set the window text.
- */
-static LRESULT DEFWND_SetTextA( HWND hwnd, LPCSTR text )
-{
-    int count;
-    WCHAR *textW;
-    WND *wndPtr;
-
-    /* check for string, as static icons, bitmaps (SS_ICON, SS_BITMAP)
-     * may have child window IDs instead of window name */
-    if (text && IS_INTRESOURCE(text))
-        return 0;
-
-    if (!text) text = "";
-    count = MultiByteToWideChar( CP_ACP, 0, text, -1, NULL, 0 );
-
-    if (!(wndPtr = WIN_GetPtr( hwnd ))) return 0;
-    if ((textW = HeapAlloc(GetProcessHeap(), 0, count * sizeof(WCHAR))))
-    {
-        HeapFree(GetProcessHeap(), 0, wndPtr->text);
-        wndPtr->text = textW;
-        MultiByteToWideChar( CP_ACP, 0, text, -1, textW, count );
-        SERVER_START_REQ( set_window_text )
-        {
-            req->handle = wine_server_user_handle( hwnd );
-            wine_server_add_data( req, textW, (count-1) * sizeof(WCHAR) );
-            wine_server_call( req );
-        }
-        SERVER_END_REQ;
-    }
-    else
-        ERR("Not enough memory for window text\n");
-    WIN_ReleasePtr( wndPtr );
-
-    USER_Driver->pSetWindowText( hwnd, textW );
-
-    return 1;
-}
-
-/***********************************************************************
- *           DEFWND_SetTextW
- *
- * Set the window text.
- */
-static LRESULT DEFWND_SetTextW( HWND hwnd, LPCWSTR text )
-{
-    WND *wndPtr;
-    int count;
-
-    /* check for string, as static icons, bitmaps (SS_ICON, SS_BITMAP)
-     * may have child window IDs instead of window name */
-    if (text && IS_INTRESOURCE(text))
-        return 0;
-
-    if (!text) text = L"";
-    count = lstrlenW(text) + 1;
-
-    if (!(wndPtr = WIN_GetPtr( hwnd ))) return 0;
-    HeapFree(GetProcessHeap(), 0, wndPtr->text);
-    if ((wndPtr->text = HeapAlloc(GetProcessHeap(), 0, count * sizeof(WCHAR))))
-    {
-        lstrcpyW( wndPtr->text, text );
-        SERVER_START_REQ( set_window_text )
-        {
-            req->handle = wine_server_user_handle( hwnd );
-            wine_server_add_data( req, wndPtr->text, (count-1) * sizeof(WCHAR) );
-            wine_server_call( req );
-        }
-        SERVER_END_REQ;
-    }
-    else
-        ERR("Not enough memory for window text\n");
-    text = wndPtr->text;
-    WIN_ReleasePtr( wndPtr );
-
-    USER_Driver->pSetWindowText( hwnd, text );
-
-    return 1;
-}
-
 /***********************************************************************
  *           DEFWND_ControlColor
  *
@@ -242,30 +159,12 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
 {
     switch(msg)
     {
-    case WM_NCPAINT:
-        return NC_HandleNCPaint( hwnd, (HRGN)wParam );
-
     case WM_NCMOUSEMOVE:
         return NC_HandleNCMouseMove( hwnd, wParam, lParam );
 
     case WM_NCMOUSELEAVE:
         return NC_HandleNCMouseLeave( hwnd );
 
-    case WM_NCHITTEST:
-        {
-            POINT pt;
-            pt.x = (short)LOWORD(lParam);
-            pt.y = (short)HIWORD(lParam);
-            return NC_HandleNCHitTest( hwnd, pt );
-        }
-
-    case WM_NCCALCSIZE:
-        NC_HandleNCCalcSize( hwnd, wParam, (RECT *)lParam );
-        break;
-
-    case WM_WINDOWPOSCHANGING:
-        return WINPOS_HandleWindowPosChanging( hwnd, (WINDOWPOS *)lParam );
-
     case WM_WINDOWPOSCHANGED:
         DEFWND_HandleWindowPosChanged( hwnd, (const WINDOWPOS *)lParam );
         break;
@@ -276,18 +175,6 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         iF10Key = iMenuSysKey = 0;
         break;
 
-    case WM_NCLBUTTONDOWN:
-        return NC_HandleNCLButtonDown( hwnd, wParam, lParam );
-
-    case WM_LBUTTONDBLCLK:
-        return NC_HandleNCLButtonDblClk( hwnd, HTCLIENT, lParam );
-
-    case WM_NCLBUTTONDBLCLK:
-        return NC_HandleNCLButtonDblClk( hwnd, wParam, lParam );
-
-    case WM_NCRBUTTONDOWN:
-        return NC_HandleNCRButtonDown( hwnd, wParam, lParam );
-
     case WM_RBUTTONUP:
         {
             POINT pt;
@@ -316,138 +203,10 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         }
         break;
 
-    case WM_CONTEXTMENU:
-        if (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CHILD)
-            SendMessageW( GetParent(hwnd), msg, (WPARAM)hwnd, lParam );
-        else
-        {
-            LONG hitcode;
-            POINT pt;
-            pt.x = (short)LOWORD(lParam);
-            pt.y = (short)HIWORD(lParam);
-            hitcode = NC_HandleNCHitTest(hwnd, pt);
-
-            /* Track system popup if click was in the caption area. */
-            if (hitcode==HTCAPTION || hitcode==HTSYSMENU)
-               TrackPopupMenu(GetSystemMenu(hwnd, FALSE),
-                               TPM_LEFTBUTTON | TPM_RIGHTBUTTON,
-                               pt.x, pt.y, 0, hwnd, NULL);
-        }
-        break;
-
-    case WM_POPUPSYSTEMMENU:
-        /* This is an undocumented message used by the windows taskbar to
-           display the system menu of windows that belong to other processes. */
-        TrackPopupMenu( GetSystemMenu(hwnd, FALSE), TPM_LEFTBUTTON|TPM_RIGHTBUTTON,
-                        (short)LOWORD(lParam), (short)HIWORD(lParam), 0, hwnd, NULL );
-        return 0;
-
-    case WM_NCACTIVATE:
-        return NC_HandleNCActivate( hwnd, wParam, lParam );
-
-    case WM_NCDESTROY:
-        {
-            WND *wndPtr = WIN_GetPtr( hwnd );
-            if (!wndPtr) return 0;
-            HeapFree( GetProcessHeap(), 0, wndPtr->text );
-            wndPtr->text = NULL;
-            HeapFree( GetProcessHeap(), 0, wndPtr->pScroll );
-            wndPtr->pScroll = NULL;
-            WIN_ReleasePtr( wndPtr );
-            return 0;
-        }
-
     case WM_PRINT:
         DEFWND_Print(hwnd, (HDC)wParam, lParam);
         return 0;
 
-    case WM_PAINTICON:
-    case WM_PAINT:
-        {
-            PAINTSTRUCT ps;
-            HDC hdc = BeginPaint( hwnd, &ps );
-            if( hdc )
-            {
-              HICON hIcon;
-              if (IsIconic(hwnd) && ((hIcon = (HICON)GetClassLongPtrW( hwnd, GCLP_HICON))) )
-              {
-                  RECT rc;
-                  int x, y;
-
-                  GetClientRect( hwnd, &rc );
-                  x = (rc.right - rc.left - GetSystemMetrics(SM_CXICON))/2;
-                  y = (rc.bottom - rc.top - GetSystemMetrics(SM_CYICON))/2;
-                  TRACE("Painting class icon: vis rect=(%s)\n",
-                        wine_dbgstr_rect(&ps.rcPaint));
-                  DrawIcon( hdc, x, y, hIcon );
-              }
-              EndPaint( hwnd, &ps );
-            }
-            return 0;
-        }
-
-    case WM_SYNCPAINT:
-        RedrawWindow ( hwnd, NULL, 0, RDW_ERASENOW | RDW_ERASE | RDW_ALLCHILDREN );
-        return 0;
-
-    case WM_SETREDRAW:
-        if (wParam) WIN_SetStyle( hwnd, WS_VISIBLE, 0 );
-        else
-        {
-            RedrawWindow( hwnd, NULL, 0, RDW_ALLCHILDREN | RDW_VALIDATE );
-            WIN_SetStyle( hwnd, 0, WS_VISIBLE );
-        }
-        return 0;
-
-    case WM_CLOSE:
-        DestroyWindow( hwnd );
-        return 0;
-
-    case WM_MOUSEACTIVATE:
-        if (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CHILD)
-        {
-            LONG ret = SendMessageW( GetParent(hwnd), WM_MOUSEACTIVATE, wParam, lParam );
-            if (ret) return ret;
-        }
-
-        /* Caption clicks are handled by NC_HandleNCLButtonDown() */
-        return ( HIWORD(lParam) == WM_LBUTTONDOWN && LOWORD(lParam) == HTCAPTION ? MA_NOACTIVATE : MA_ACTIVATE );
-
-    case WM_ACTIVATE:
-        /* The default action in Windows is to set the keyboard focus to
-         * the window, if it's being activated and not minimized */
-        if (LOWORD(wParam) != WA_INACTIVE) {
-            if (!IsIconic(hwnd)) SetFocus(hwnd);
-        }
-        break;
-
-    case WM_MOUSEWHEEL:
-        if (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CHILD)
-            return SendMessageW( GetParent(hwnd), WM_MOUSEWHEEL, wParam, lParam );
-        break;
-
-    case WM_ERASEBKGND:
-    case WM_ICONERASEBKGND:
-        {
-            RECT rect;
-            HDC hdc = (HDC)wParam;
-            HBRUSH hbr = (HBRUSH)GetClassLongPtrW( hwnd, GCLP_HBRBACKGROUND );
-            if (!hbr) return 0;
-
-            if (GetClassLongW( hwnd, GCL_STYLE ) & CS_PARENTDC)
-            {
-                /* can't use GetClipBox with a parent DC or we fill the whole parent */
-                GetClientRect( hwnd, &rect );
-                DPtoLP( hdc, (LPPOINT)&rect, 2 );
-            }
-            else GetClipBox( hdc, &rect );
-            FillRect( hdc, &rect, hbr );
-            return 1;
-        }
-
-    case WM_GETDLGCODE:
-        return 0;
-
     case WM_CTLCOLORMSGBOX:
     case WM_CTLCOLOREDIT:
     case WM_CTLCOLORLISTBOX:
@@ -460,21 +219,6 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
     case WM_CTLCOLOR:
         return (LRESULT)DEFWND_ControlColor( (HDC)wParam, HIWORD(lParam) );
 
-    case WM_SETCURSOR:
-        if (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CHILD)
-        {
-            /* with the exception of the border around a resizable wnd,
-             * give the parent first chance to set the cursor */
-            if ((LOWORD(lParam) < HTSIZEFIRST) || (LOWORD(lParam) > HTSIZELAST))
-            {
-                HWND parent = GetParent( hwnd );
-                if (parent != GetDesktopWindow() &&
-                    SendMessageW( parent, WM_SETCURSOR, wParam, lParam )) return TRUE;
-            }
-        }
-        NC_HandleSetCursor( hwnd, wParam, lParam );
-        break;
-
     case WM_SYSCOMMAND:
         return NC_HandleSysCommand( hwnd, wParam, lParam );
 
@@ -496,7 +240,7 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
 
             if( wParam == VK_F4 )       /* try to close the window */
             {
-                HWND top = GetAncestor( hwnd, GA_ROOT );
+                HWND top = NtUserGetAncestor( hwnd, GA_ROOT );
                 if (!(GetClassLongW( top, GCL_STYLE ) & CS_NOCLOSE))
                     PostMessageW( top, WM_SYSCOMMAND, SC_CLOSE, 0 );
             }
@@ -516,7 +260,7 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         /* Press and release F10 or ALT */
         if (((wParam == VK_MENU || wParam == VK_LMENU || wParam == VK_RMENU)
              && iMenuSysKey) || ((wParam == VK_F10) && iF10Key))
-              SendMessageW( GetAncestor( hwnd, GA_ROOT ), WM_SYSCOMMAND, SC_KEYMENU, 0L );
+              SendMessageW( NtUserGetAncestor( hwnd, GA_ROOT ), WM_SYSCOMMAND, SC_KEYMENU, 0L );
         iMenuSysKey = iF10Key = 0;
         break;
 
@@ -562,14 +306,13 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
             }
             else pWnd->flags |= WIN_NEEDS_SHOW_OWNEDPOPUP;
             WIN_ReleasePtr( pWnd );
-            ShowWindow( hwnd, wParam ? SW_SHOWNOACTIVATE : SW_HIDE );
+            NtUserShowWindow( hwnd, wParam ? SW_SHOWNOACTIVATE : SW_HIDE );
             break;
         }
 
     case WM_CANCELMODE:
         iMenuSysKey = 0;
-        MENU_EndMenu( hwnd );
-        if (GetCapture() == hwnd) ReleaseCapture();
+        NtUserMessageCall( hwnd, msg, wParam, lParam, 0, NtUserDefWindowProc, FALSE );
         break;
 
     case WM_VKEYTOITEM:
@@ -607,81 +350,6 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
     case WM_QUERYENDSESSION:
         return 1;
 
-    case WM_SETICON:
-        {
-            HICON ret;
-            WND *wndPtr = WIN_GetPtr( hwnd );
-
-            switch(wParam)
-            {
-            case ICON_SMALL:
-                ret = wndPtr->hIconSmall;
-                if (ret && !lParam && wndPtr->hIcon)
-                {
-                    wndPtr->hIconSmall2 = CopyImage( wndPtr->hIcon, IMAGE_ICON,
-                                                     GetSystemMetrics( SM_CXSMICON ),
-                                                     GetSystemMetrics( SM_CYSMICON ), 0 );
-                }
-                else if (lParam && wndPtr->hIconSmall2)
-                {
-                    DestroyIcon( wndPtr->hIconSmall2 );
-                    wndPtr->hIconSmall2 = NULL;
-                }
-                wndPtr->hIconSmall = (HICON)lParam;
-                break;
-            case ICON_BIG:
-                ret = wndPtr->hIcon;
-                if (wndPtr->hIconSmall2)
-                {
-                    DestroyIcon( wndPtr->hIconSmall2 );
-                    wndPtr->hIconSmall2 = NULL;
-                }
-                if (lParam && !wndPtr->hIconSmall)
-                {
-                    wndPtr->hIconSmall2 = CopyImage( (HICON)lParam, IMAGE_ICON,
-                                                     GetSystemMetrics( SM_CXSMICON ),
-                                                     GetSystemMetrics( SM_CYSMICON ), 0 );
-                }
-                wndPtr->hIcon = (HICON)lParam;
-                break;
-            default:
-                ret = 0;
-                break;
-            }
-            WIN_ReleasePtr( wndPtr );
-
-            USER_Driver->pSetWindowIcon( hwnd, wParam, (HICON)lParam );
-
-            if( (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CAPTION) == WS_CAPTION )
-                NC_HandleNCPaint( hwnd , (HRGN)1 );  /* Repaint caption */
-
-            return (LRESULT)ret;
-        }
-
-    case WM_GETICON:
-        {
-            HICON ret;
-            WND *wndPtr = WIN_GetPtr( hwnd );
-
-            switch(wParam)
-            {
-            case ICON_SMALL:
-                ret = wndPtr->hIconSmall;
-                break;
-            case ICON_BIG:
-                ret = wndPtr->hIcon;
-                break;
-            case ICON_SMALL2:
-                ret = wndPtr->hIconSmall ? wndPtr->hIconSmall : wndPtr->hIconSmall2;
-                break;
-            default:
-                ret = 0;
-                break;
-            }
-            WIN_ReleasePtr( wndPtr );
-            return (LRESULT)ret;
-        }
-
     case WM_HELP:
         SendMessageW( GetParent(hwnd), msg, wParam, lParam );
         break;
@@ -691,8 +359,8 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
         {
             STYLESTRUCT *style = (STYLESTRUCT *)lParam;
             if ((style->styleOld ^ style->styleNew) & (WS_CAPTION|WS_THICKFRAME|WS_VSCROLL|WS_HSCROLL))
-                SetWindowPos( hwnd, 0, 0, 0, 0, 0, SWP_FRAMECHANGED | SWP_NOACTIVATE | SWP_NOZORDER |
-                              SWP_NOSIZE | SWP_NOMOVE | SWP_NOCLIENTSIZE | SWP_NOCLIENTMOVE );
+                NtUserSetWindowPos( hwnd, 0, 0, 0, 0, 0, SWP_FRAMECHANGED | SWP_NOACTIVATE | SWP_NOZORDER |
+                                    SWP_NOSIZE | SWP_NOMOVE | SWP_NOCLIENTSIZE | SWP_NOCLIENTMOVE );
         }
         break;
 
@@ -705,29 +373,6 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
                 SendMessageW( parent, msg, wParam, lParam );
             break;
         }
-    case WM_KEYF1:
-        {
-            HELPINFO hi;
-
-            hi.cbSize = sizeof(HELPINFO);
-            GetCursorPos( &hi.MousePos );
-            if (MENU_IsMenuActive())
-            {
-                hi.iContextType = HELPINFO_MENUITEM;
-                hi.hItemHandle = MENU_IsMenuActive();
-                hi.iCtrlId = MenuItemFromPoint( hwnd, hi.hItemHandle, hi.MousePos );
-                hi.dwContextId = GetMenuContextHelpId( hi.hItemHandle );
-            }
-            else
-            {
-                hi.iContextType = HELPINFO_WINDOW;
-                hi.hItemHandle = hwnd;
-                hi.iCtrlId = GetWindowLongPtrA( hwnd, GWLP_ID );
-                hi.dwContextId = GetWindowContextHelpId( hwnd );
-            }
-            SendMessageW( hwnd, WM_HELP, 0, (LPARAM)&hi );
-            break;
-        }
 
     case WM_INPUTLANGCHANGEREQUEST:
         NtUserActivateKeyboardLayout( (HKL)lParam, 0 );
@@ -748,6 +393,9 @@ static LRESULT DEFWND_DefWinProc( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lPa
             break;
         }
 
+    default:
+        return NtUserMessageCall( hwnd, msg, wParam, lParam, 0, NtUserDefWindowProc, FALSE );
+
     }
 
     return 0;
@@ -802,8 +450,7 @@ LRESULT WINAPI DefWindowProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam
         {
             CREATESTRUCTA *cs = (CREATESTRUCTA *)lParam;
 
-            DEFWND_SetTextA( hwnd, cs->lpszName );
-            result = 1;
+            result = NtUserMessageCall( hwnd, msg, wParam, lParam, 0, NtUserDefWindowProc, TRUE );
 
             if(cs->style & (WS_HSCROLL | WS_VSCROLL))
             {
@@ -838,11 +485,7 @@ LRESULT WINAPI DefWindowProcA( HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam
         break;
 
     case WM_SETTEXT:
-        if (!DEFWND_SetTextA( hwnd, (LPCSTR)lParam ))
-            break;
-        if( (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CAPTION) == WS_CAPTION )
-            NC_HandleNCPaint( hwnd , (HRGN)1 );  /* Repaint caption */
-        result = 1; /* success. FIXME: check text length */
+        result = NtUserMessageCall( hwnd, msg, wParam, lParam, 0, NtUserDefWindowProc, TRUE );
         break;
 
     case WM_IME_CHAR:
@@ -987,8 +630,7 @@ LRESULT WINAPI DefWindowProcW(
         {
             CREATESTRUCTW *cs = (CREATESTRUCTW *)lParam;
 
-            DEFWND_SetTextW( hwnd, cs->lpszName );
-            result = 1;
+            result = NtUserMessageCall( hwnd, msg, wParam, lParam, 0, NtUserDefWindowProc, FALSE );
 
             if(cs->style & (WS_HSCROLL | WS_VSCROLL))
             {
@@ -1019,14 +661,6 @@ LRESULT WINAPI DefWindowProcW(
         }
         break;
 
-    case WM_SETTEXT:
-        if (!DEFWND_SetTextW( hwnd, (LPCWSTR)lParam ))
-            break;
-        if( (GetWindowLongW( hwnd, GWL_STYLE ) & WS_CAPTION) == WS_CAPTION )
-            NC_HandleNCPaint( hwnd , (HRGN)1 );  /* Repaint caption */
-        result = 1; /* success. FIXME: check text length */
-        break;
-
     case WM_IME_CHAR:
         PostMessageW( hwnd, WM_CHAR, wParam, lParam );
         break;
