@@ -26,33 +26,26 @@
 #include "windef.h"
 #include "winbase.h"
 #include "wingdi.h"
+#include "winioctl.h"
 #include "winnls.h"
 #include "winreg.h"
 #include "winuser.h"
-#include "setupapi.h"
-#include "ddk/hidsdi.h"
+#include "ddk/hidclass.h"
 #include "wine/debug.h"
 #include "wine/server.h"
 #include "wine/hid.h"
 
 #include "user_private.h"
 
-#include "initguid.h"
-#include "devpkey.h"
-#include "ntddmou.h"
-#include "ntddkbd.h"
-
 WINE_DEFAULT_DEBUG_CHANNEL(rawinput);
 
-DEFINE_DEVPROPKEY(DEVPROPKEY_HID_HANDLE, 0xbc62e415, 0xf4fe, 0x405c, 0x8e, 0xda, 0x63, 0x6f, 0xb5, 0x9f, 0x08, 0x98, 2);
-
 struct device
 {
-    SP_DEVICE_INTERFACE_DETAIL_DATA_W *detail;
+    WCHAR *path;
     HANDLE file;
     HANDLE handle;
     RID_DEVICE_INFO info;
-    PHIDP_PREPARSED_DATA data;
+    struct hid_preparsed_data *data;
 };
 
 static struct device *rawinput_devices;
@@ -94,160 +87,236 @@ static BOOL array_reserve(void **elements, unsigned int *capacity, unsigned int
     return TRUE;
 }
 
-static struct device *add_device(HDEVINFO set, SP_DEVICE_INTERFACE_DATA *iface)
+static ULONG query_reg_value( HKEY hkey, const WCHAR *name,
+                              KEY_VALUE_PARTIAL_INFORMATION *info, ULONG size )
 {
-    SP_DEVINFO_DATA device_data = {sizeof(device_data)};
-    SP_DEVICE_INTERFACE_DETAIL_DATA_W *detail;
+    unsigned int name_size = name ? lstrlenW( name ) * sizeof(WCHAR) : 0;
+    UNICODE_STRING nameW = { name_size, name_size, (WCHAR *)name };
+
+    if (NtQueryValueKey( hkey, &nameW, KeyValuePartialInformation,
+                         info, size, &size ))
+        return 0;
+
+    return size - FIELD_OFFSET(KEY_VALUE_PARTIAL_INFORMATION, Data);
+}
+
+static struct device *add_device( HKEY key, DWORD type )
+{
+    static const WCHAR symbolic_linkW[] = {'S','y','m','b','o','l','i','c','L','i','n','k',0};
+    char value_buffer[4096];
+    KEY_VALUE_PARTIAL_INFORMATION *value = (KEY_VALUE_PARTIAL_INFORMATION *)value_buffer;
+    static const RID_DEVICE_INFO_KEYBOARD keyboard_info = {0, 0, 1, 12, 3, 101};
+    static const RID_DEVICE_INFO_MOUSE mouse_info = {1, 5, 0, FALSE};
+    struct hid_preparsed_data *preparsed = NULL;
+    HID_COLLECTION_INFORMATION hid_info;
     struct device *device = NULL;
+    RID_DEVICE_INFO info;
+    IO_STATUS_BLOCK io;
+    WCHAR *path, *pos;
+    NTSTATUS status;
+    unsigned int i;
     UINT32 handle;
     HANDLE file;
-    DWORD i, size, type;
 
-    SetupDiGetDeviceInterfaceDetailW(set, iface, NULL, 0, &size, &device_data);
-    if (GetLastError() != ERROR_INSUFFICIENT_BUFFER)
+    if (!query_reg_value( key, symbolic_linkW, value, sizeof(value_buffer) ))
     {
-        ERR("Failed to get device path, error %#x.\n", GetLastError());
-        return FALSE;
-    }
-
-    if (!SetupDiGetDevicePropertyW(set, &device_data, &DEVPROPKEY_HID_HANDLE, &type, (BYTE *)&handle, sizeof(handle), NULL, 0) ||
-        type != DEVPROP_TYPE_UINT32)
-    {
-        ERR("Failed to get device handle, error %#x.\n", GetLastError());
+        ERR( "failed to get symbolic link value\n" );
         return NULL;
     }
 
-    if (!(detail = malloc(size)))
-    {
-        ERR("Failed to allocate memory.\n");
-        return FALSE;
-    }
-    detail->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA_W);
-    SetupDiGetDeviceInterfaceDetailW(set, iface, detail, size, NULL, NULL);
+    if (!(path = malloc( value->DataLength + sizeof(WCHAR) )))
+        return NULL;
+    memcpy( path, value->Data, value->DataLength );
+    path[value->DataLength / sizeof(WCHAR)] = 0;
 
-    file = CreateFileW(detail->DevicePath, GENERIC_READ | GENERIC_WRITE,
-            FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0);
+    /* upper case everything but the GUID */
+    for (pos = path; *pos && *pos != '{'; pos++) *pos = towupper(*pos);
+
+    file = CreateFileW( path, GENERIC_READ | GENERIC_WRITE,
+                        FILE_SHARE_READ | FILE_SHARE_WRITE, NULL, OPEN_EXISTING, 0, 0 );
     if (file == INVALID_HANDLE_VALUE)
     {
-        ERR("Failed to open device file %s, error %u.\n", debugstr_w(detail->DevicePath), GetLastError());
-        free(detail);
+        ERR( "Failed to open device file %s, error %lu.\n", debugstr_w(path), GetLastError() );
+        free( path );
         return NULL;
     }
 
+    status = NtDeviceIoControlFile( file, NULL, NULL, NULL, &io,
+                                    IOCTL_HID_GET_WINE_RAWINPUT_HANDLE,
+                                    NULL, 0, &handle, sizeof(handle) );
+    if (status)
+    {
+        ERR( "Failed to get raw input handle, status %#lx.\n", status );
+        goto fail;
+    }
+
+    memset( &info, 0, sizeof(info) );
+    info.cbSize = sizeof(info);
+    info.dwType = type;
+
+    switch (type)
+    {
+    case RIM_TYPEHID:
+        status = NtDeviceIoControlFile( file, NULL, NULL, NULL, &io,
+                                        IOCTL_HID_GET_COLLECTION_INFORMATION,
+                                        NULL, 0, &hid_info, sizeof(hid_info) );
+        if (status)
+        {
+            ERR( "Failed to get collection information, status %#lx.\n", status );
+            goto fail;
+        }
+
+        info.hid.dwVendorId = hid_info.VendorID;
+        info.hid.dwProductId = hid_info.ProductID;
+        info.hid.dwVersionNumber = hid_info.VersionNumber;
+
+        if (!(preparsed = malloc( hid_info.DescriptorSize )))
+        {
+            ERR( "Failed to allocate memory.\n" );
+            goto fail;
+        }
+
+        status = NtDeviceIoControlFile( file, NULL, NULL, NULL, &io,
+                                        IOCTL_HID_GET_COLLECTION_DESCRIPTOR,
+                                        NULL, 0, preparsed, hid_info.DescriptorSize );
+        if (status)
+        {
+            ERR( "Failed to get collection descriptor, status %#lx.\n", status );
+            goto fail;
+        }
+
+        info.hid.usUsagePage = preparsed->usage_page;
+        info.hid.usUsage = preparsed->usage;
+        break;
+
+    case RIM_TYPEMOUSE:
+        info.mouse = mouse_info;
+        break;
+
+    case RIM_TYPEKEYBOARD:
+        info.keyboard = keyboard_info;
+        break;
+    }
+
     for (i = 0; i < rawinput_devices_count && !device; ++i)
         if (rawinput_devices[i].handle == UlongToHandle(handle))
             device = rawinput_devices + i;
 
     if (device)
     {
-        TRACE("Updating device %x / %s.\n", handle, debugstr_w(detail->DevicePath));
-        HidD_FreePreparsedData(device->data);
+        TRACE( "Updating device %#x / %s.\n", handle, debugstr_w(path) );
+        free(device->data);
         CloseHandle(device->file);
-        free(device->detail);
+        free( device->path );
     }
     else if (array_reserve((void **)&rawinput_devices, &rawinput_devices_max,
                            rawinput_devices_count + 1, sizeof(*rawinput_devices)))
     {
         device = &rawinput_devices[rawinput_devices_count++];
-        TRACE("Adding device %x / %s.\n", handle, debugstr_w(detail->DevicePath));
+        TRACE( "Adding device %#x / %s.\n", handle, debugstr_w(path) );
     }
     else
     {
         ERR("Failed to allocate memory.\n");
-        CloseHandle(file);
-        free(detail);
-        return NULL;
+        goto fail;
     }
 
-    device->detail = detail;
+    device->path = path;
     device->file = file;
     device->handle = ULongToHandle(handle);
-    device->info.cbSize = sizeof(RID_DEVICE_INFO);
-    device->data = NULL;
+    device->info = info;
+    device->data = preparsed;
 
     return device;
+
+fail:
+    free( preparsed );
+    CloseHandle( file );
+    free( path );
+    return NULL;
 }
 
-static void find_devices(void)
+static HKEY reg_open_key( HKEY root, const WCHAR *name, ULONG name_len )
+{
+    UNICODE_STRING nameW = { name_len, name_len, (WCHAR *)name };
+    OBJECT_ATTRIBUTES attr;
+    HANDLE ret;
+
+    attr.Length = sizeof(attr);
+    attr.RootDirectory = root;
+    attr.ObjectName = &nameW;
+    attr.Attributes = 0;
+    attr.SecurityDescriptor = NULL;
+    attr.SecurityQualityOfService = NULL;
+
+    if (NtOpenKeyEx( &ret, MAXIMUM_ALLOWED, &attr, 0 )) return 0;
+    return ret;
+}
+
+static const WCHAR device_classesW[] = L"\\Registry\\Machine\\System\\CurrentControlSet\\Control\\DeviceClasses\\";
+static const WCHAR guid_devinterface_hidW[] = L"{4d1e55b2-f16f-11cf-88cb-001111000030}";
+static const WCHAR guid_devinterface_keyboardW[] = L"{884b96c3-56ef-11d1-bc8c-00a0c91405dd}";
+static const WCHAR guid_devinterface_mouseW[] = L"{378de44c-56ef-11d1-bc8c-00a0c91405dd}";
+
+static void enumerate_devices( DWORD type, const WCHAR *class )
+{
+    WCHAR buffer[1024];
+    KEY_NODE_INFORMATION *subkey_info = (void *)buffer;
+    HKEY class_key, device_key, iface_key;
+    unsigned int i, j;
+    DWORD size;
+
+    wcscpy( buffer, device_classesW );
+    wcscat( buffer, class );
+    if (!(class_key = reg_open_key( NULL, buffer, wcslen( buffer ) * sizeof(WCHAR) )))
+        return;
+
+    for (i = 0; !NtEnumerateKey( class_key, i, KeyNodeInformation, buffer, sizeof(buffer), &size ); ++i)
+    {
+        if (!(device_key = reg_open_key( class_key, subkey_info->Name, subkey_info->NameLength )))
+        {
+            ERR( "failed to open %s\n", debugstr_wn(subkey_info->Name, subkey_info->NameLength / sizeof(WCHAR)) );
+            continue;
+        }
+
+        for (j = 0; !NtEnumerateKey( device_key, j, KeyNodeInformation, buffer, sizeof(buffer), &size ); ++j)
+        {
+            if (!(iface_key = reg_open_key( device_key, subkey_info->Name, subkey_info->NameLength )))
+            {
+                ERR( "failed to open %s\n", debugstr_wn(subkey_info->Name, subkey_info->NameLength / sizeof(WCHAR)) );
+                continue;
+            }
+
+            add_device( iface_key, type );
+            NtClose( iface_key );
+        }
+
+        NtClose( device_key );
+    }
+
+    NtClose( class_key );
+}
+
+void rawinput_update_device_list(void)
 {
-    SP_DEVICE_INTERFACE_DATA iface = { sizeof(iface) };
-    struct device *device;
-    HIDD_ATTRIBUTES attr;
-    HIDP_CAPS caps;
-    GUID hid_guid;
-    HDEVINFO set;
     DWORD idx;
 
-    HidD_GetHidGuid(&hid_guid);
+    TRACE("\n");
 
     EnterCriticalSection(&rawinput_devices_cs);
 
     /* destroy previous list */
     for (idx = 0; idx < rawinput_devices_count; ++idx)
     {
-        HidD_FreePreparsedData(rawinput_devices[idx].data);
+        free(rawinput_devices[idx].data);
         CloseHandle(rawinput_devices[idx].file);
-        free(rawinput_devices[idx].detail);
+        free( rawinput_devices[idx].path );
     }
     rawinput_devices_count = 0;
 
-    set = SetupDiGetClassDevsW(&hid_guid, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);
-
-    for (idx = 0; SetupDiEnumDeviceInterfaces(set, NULL, &hid_guid, idx, &iface); ++idx)
-    {
-        if (!(device = add_device(set, &iface)))
-            continue;
-
-        attr.Size = sizeof(HIDD_ATTRIBUTES);
-        if (!HidD_GetAttributes(device->file, &attr))
-            WARN("Failed to get attributes.\n");
-
-        device->info.dwType = RIM_TYPEHID;
-        device->info.hid.dwVendorId = attr.VendorID;
-        device->info.hid.dwProductId = attr.ProductID;
-        device->info.hid.dwVersionNumber = attr.VersionNumber;
-
-        if (!HidD_GetPreparsedData(device->file, &device->data))
-            WARN("Failed to get preparsed data.\n");
-
-        if (!HidP_GetCaps(device->data, &caps))
-            WARN("Failed to get caps.\n");
-
-        device->info.hid.usUsagePage = caps.UsagePage;
-        device->info.hid.usUsage = caps.Usage;
-    }
-
-    SetupDiDestroyDeviceInfoList(set);
-
-    set = SetupDiGetClassDevsW(&GUID_DEVINTERFACE_MOUSE, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);
-
-    for (idx = 0; SetupDiEnumDeviceInterfaces(set, NULL, &GUID_DEVINTERFACE_MOUSE, idx, &iface); ++idx)
-    {
-        static const RID_DEVICE_INFO_MOUSE mouse_info = {1, 5, 0, FALSE};
-
-        if (!(device = add_device(set, &iface)))
-            continue;
-
-        device->info.dwType = RIM_TYPEMOUSE;
-        device->info.mouse = mouse_info;
-    }
-
-    SetupDiDestroyDeviceInfoList(set);
-
-    set = SetupDiGetClassDevsW(&GUID_DEVINTERFACE_KEYBOARD, NULL, NULL, DIGCF_DEVICEINTERFACE | DIGCF_PRESENT);
-
-    for (idx = 0; SetupDiEnumDeviceInterfaces(set, NULL, &GUID_DEVINTERFACE_KEYBOARD, idx, &iface); ++idx)
-    {
-        static const RID_DEVICE_INFO_KEYBOARD keyboard_info = {0, 0, 1, 12, 3, 101};
-
-        if (!(device = add_device(set, &iface)))
-            continue;
-
-        device->info.dwType = RIM_TYPEKEYBOARD;
-        device->info.keyboard = keyboard_info;
-    }
-
-    SetupDiDestroyDeviceInfoList(set);
+    enumerate_devices( RIM_TYPEHID, guid_devinterface_hidW );
+    enumerate_devices( RIM_TYPEMOUSE, guid_devinterface_mouseW );
+    enumerate_devices( RIM_TYPEKEYBOARD, guid_devinterface_keyboardW );
 
     LeaveCriticalSection(&rawinput_devices_cs);
 }
@@ -259,7 +328,7 @@ static struct device *find_device_from_handle(HANDLE handle)
     for (i = 0; i < rawinput_devices_count; ++i)
         if (rawinput_devices[i].handle == handle)
             return rawinput_devices + i;
-    find_devices();
+    rawinput_update_device_list();
     for (i = 0; i < rawinput_devices_count; ++i)
         if (rawinput_devices[i].handle == handle)
             return rawinput_devices + i;
@@ -282,7 +351,7 @@ BOOL rawinput_device_get_usages(HANDLE handle, USAGE *usage_page, USAGE *usage)
 }
 
 
-struct rawinput_thread_data *rawinput_thread_data(void)
+struct rawinput_thread_data * WINAPI rawinput_thread_data(void)
 {
     struct user_thread_info *thread_info = get_user_thread_info();
     struct rawinput_thread_data *data = thread_info->rawinput;
@@ -416,7 +485,8 @@ BOOL rawinput_from_hardware_message(RAWINPUT *rawinput, const struct hardware_ms
  */
 UINT WINAPI GetRawInputDeviceList(RAWINPUTDEVICELIST *devices, UINT *device_count, UINT size)
 {
-    UINT i;
+    static UINT last_check;
+    UINT i, ticks = GetTickCount();
 
     TRACE("devices %p, device_count %p, size %u.\n", devices, device_count, size);
 
@@ -432,7 +502,11 @@ UINT WINAPI GetRawInputDeviceList(RAWINPUTDEVICELIST *devices, UINT *device_coun
         return ~0U;
     }
 
-    find_devices();
+    if (ticks - last_check > 2000)
+    {
+        last_check = ticks;
+        rawinput_update_device_list();
+    }
 
     if (!devices)
     {
@@ -459,7 +533,7 @@ UINT WINAPI GetRawInputDeviceList(RAWINPUTDEVICELIST *devices, UINT *device_coun
 /***********************************************************************
  *              RegisterRawInputDevices   (USER32.@)
  */
-BOOL WINAPI DECLSPEC_HOTPATCH RegisterRawInputDevices(RAWINPUTDEVICE *devices, UINT device_count, UINT size)
+BOOL WINAPI DECLSPEC_HOTPATCH RegisterRawInputDevices(const RAWINPUTDEVICE *devices, UINT device_count, UINT size)
 {
     struct rawinput_device *d;
     BOOL ret;
@@ -495,11 +569,11 @@ BOOL WINAPI DECLSPEC_HOTPATCH RegisterRawInputDevices(RAWINPUTDEVICE *devices, U
 
     for (i = 0; i < device_count; ++i)
     {
-        TRACE("device %u: page %#x, usage %#x, flags %#x, target %p.\n",
+        TRACE("device %u: page %#x, usage %#x, flags %#lx, target %p.\n",
                 i, devices[i].usUsagePage, devices[i].usUsage,
                 devices[i].dwFlags, devices[i].hwndTarget);
         if (devices[i].dwFlags & ~(RIDEV_REMOVE|RIDEV_NOLEGACY|RIDEV_INPUTSINK|RIDEV_DEVNOTIFY))
-            FIXME("Unhandled flags %#x for device %u.\n", devices[i].dwFlags, i);
+            FIXME("Unhandled flags %#lx for device %u.\n", devices[i].dwFlags, i);
 
         d[i].usage_page = devices[i].usUsagePage;
         d[i].usage = devices[i].usUsage;
@@ -519,161 +593,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH RegisterRawInputDevices(RAWINPUTDEVICE *devices, U
     return ret;
 }
 
-/***********************************************************************
- *              GetRawInputData   (USER32.@)
- */
-UINT WINAPI GetRawInputData(HRAWINPUT rawinput, UINT command, void *data, UINT *data_size, UINT header_size)
-{
-    struct rawinput_thread_data *thread_data = rawinput_thread_data();
-    UINT size;
-
-    TRACE("rawinput %p, command %#x, data %p, data_size %p, header_size %u.\n",
-            rawinput, command, data, data_size, header_size);
-
-    if (!rawinput || thread_data->hw_id != (UINT_PTR)rawinput)
-    {
-        SetLastError(ERROR_INVALID_HANDLE);
-        return ~0U;
-    }
-
-    if (header_size != sizeof(RAWINPUTHEADER))
-    {
-        WARN("Invalid structure size %u.\n", header_size);
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return ~0U;
-    }
-
-    switch (command)
-    {
-    case RID_INPUT:
-        size = thread_data->buffer->header.dwSize;
-        break;
-    case RID_HEADER:
-        size = sizeof(RAWINPUTHEADER);
-        break;
-    default:
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return ~0U;
-    }
-
-    if (!data)
-    {
-        *data_size = size;
-        return 0;
-    }
-
-    if (*data_size < size)
-    {
-        SetLastError(ERROR_INSUFFICIENT_BUFFER);
-        return ~0U;
-    }
-    memcpy(data, thread_data->buffer, size);
-    return size;
-}
-
-#ifdef _WIN64
-typedef RAWINPUT RAWINPUT64;
-#else
-typedef struct
-{
-    RAWINPUTHEADER header;
-    char pad[8];
-    union {
-        RAWMOUSE    mouse;
-        RAWKEYBOARD keyboard;
-        RAWHID      hid;
-    } data;
-} RAWINPUT64;
-#endif
-
-/***********************************************************************
- *              GetRawInputBuffer   (USER32.@)
- */
-UINT WINAPI DECLSPEC_HOTPATCH GetRawInputBuffer(RAWINPUT *data, UINT *data_size, UINT header_size)
-{
-    struct hardware_msg_data *msg_data;
-    struct rawinput_thread_data *thread_data;
-    RAWINPUT *rawinput;
-    UINT count = 0, remaining, rawinput_size, next_size, overhead;
-    BOOL is_wow64;
-    int i;
-
-    if (IsWow64Process( GetCurrentProcess(), &is_wow64 ) && is_wow64)
-        rawinput_size = sizeof(RAWINPUT64);
-    else
-        rawinput_size = sizeof(RAWINPUT);
-    overhead = rawinput_size - sizeof(RAWINPUT);
-
-    if (header_size != sizeof(RAWINPUTHEADER))
-    {
-        WARN("Invalid structure size %u.\n", header_size);
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return ~0U;
-    }
-
-    if (!data_size)
-    {
-        SetLastError(ERROR_INVALID_PARAMETER);
-        return ~0U;
-    }
-
-    if (!data)
-    {
-        TRACE("data %p, data_size %p (%u), header_size %u\n", data, data_size, *data_size, header_size);
-        SERVER_START_REQ( get_rawinput_buffer )
-        {
-            req->rawinput_size = rawinput_size;
-            req->buffer_size = 0;
-            if (wine_server_call( req )) return ~0U;
-            *data_size = reply->next_size;
-        }
-        SERVER_END_REQ;
-        return 0;
-    }
-
-    if (!(thread_data = rawinput_thread_data())) return ~0U;
-    rawinput = thread_data->buffer;
-
-    /* first RAWINPUT block in the buffer is used for WM_INPUT message data */
-    msg_data = (struct hardware_msg_data *)NEXTRAWINPUTBLOCK(rawinput);
-    SERVER_START_REQ( get_rawinput_buffer )
-    {
-        req->rawinput_size = rawinput_size;
-        req->buffer_size = *data_size;
-        wine_server_set_reply( req, msg_data, RAWINPUT_BUFFER_SIZE - rawinput->header.dwSize );
-        if (wine_server_call( req )) return ~0U;
-        next_size = reply->next_size;
-        count = reply->count;
-    }
-    SERVER_END_REQ;
-
-    remaining = *data_size;
-    for (i = 0; i < count; ++i)
-    {
-        data->header.dwSize = remaining;
-        if (!rawinput_from_hardware_message(data, msg_data)) break;
-        if (overhead) memmove((char *)&data->data + overhead, &data->data,
-                              data->header.dwSize - sizeof(RAWINPUTHEADER));
-        data->header.dwSize += overhead;
-        remaining -= data->header.dwSize;
-        data = NEXTRAWINPUTBLOCK(data);
-        msg_data = (struct hardware_msg_data *)((char *)msg_data + msg_data->size);
-    }
-
-    if (count == 0 && next_size == 0) *data_size = 0;
-    else if (next_size == 0) next_size = rawinput_size;
-
-    if (next_size && *data_size <= next_size)
-    {
-        SetLastError(ERROR_INSUFFICIENT_BUFFER);
-        *data_size = next_size;
-        count = ~0U;
-    }
-
-    if (count) TRACE("data %p, data_size %p (%u), header_size %u, count %u\n", data, data_size, *data_size, header_size, count);
-    return count;
-}
-
 /***********************************************************************
  *              GetRawInputDeviceInfoA   (USER32.@)
  */
@@ -740,8 +659,8 @@ UINT WINAPI GetRawInputDeviceInfoW(HANDLE handle, UINT command, void *data, UINT
     switch (command)
     {
     case RIDI_DEVICENAME:
-        if ((len = wcslen(device->detail->DevicePath) + 1) <= data_len && data)
-            memcpy(data, device->detail->DevicePath, len * sizeof(WCHAR));
+        if ((len = wcslen( device->path ) + 1) <= data_len && data)
+            memcpy( data, device->path, len * sizeof(WCHAR) );
         *data_size = len;
         break;
 
@@ -752,12 +671,12 @@ UINT WINAPI GetRawInputDeviceInfoW(HANDLE handle, UINT command, void *data, UINT
         break;
 
     case RIDI_PREPARSEDDATA:
-        if (!(preparsed = (struct hid_preparsed_data *)device->data)) len = 0;
+        if (!(preparsed = device->data)) len = 0;
         else len = preparsed->caps_size + FIELD_OFFSET(struct hid_preparsed_data, value_caps[0]) +
                    preparsed->number_link_collection_nodes * sizeof(struct hid_collection_node);
 
-        if (device->data && len <= data_len && data)
-            memcpy(data, device->data, len);
+        if (preparsed && len <= data_len && data)
+            memcpy(data, preparsed, len);
         *data_size = len;
         break;
 
