@@ -464,40 +464,22 @@ void make_gdi_object_system( HGDIOBJ handle, BOOL set)
 /******************************************************************************
  *      get_default_fonts
  */
-static const struct DefaultFontInfo* get_default_fonts(UINT charset)
+static const struct DefaultFontInfo* get_default_fonts(void)
 {
-        unsigned int n;
+    unsigned int n;
+    CHARSETINFO csi;
 
-        for(n = 0; n < ARRAY_SIZE( default_fonts ); n++)
-        {
-                if ( default_fonts[n].charset == charset )
-                        return &default_fonts[n];
-        }
+    if (ansi_cp.CodePage == CP_UTF8) return &default_fonts[0];
 
-        FIXME( "unhandled charset 0x%08x - use ANSI_CHARSET for default stock objects\n", charset );
-        return &default_fonts[0];
-}
-
-
-/******************************************************************************
- *      get_default_charset    (internal)
- *
- * get the language-dependent charset that can handle CP_ACP correctly.
- */
-static UINT get_default_charset( void )
-{
-    CHARSETINFO     csi;
-    UINT    uACP;
-
-    uACP = get_acp();
     csi.ciCharset = ANSI_CHARSET;
-    if ( !translate_charset_info( ULongToPtr(uACP), &csi, TCI_SRCCODEPAGE ) )
-    {
-        FIXME( "unhandled codepage %u - use ANSI_CHARSET for default stock objects\n", uACP );
-        return ANSI_CHARSET;
-    }
+    translate_charset_info( ULongToPtr(ansi_cp.CodePage), &csi, TCI_SRCCODEPAGE );
 
-    return csi.ciCharset;
+    for(n = 0; n < ARRAY_SIZE( default_fonts ); n++)
+        if ( default_fonts[n].charset == csi.ciCharset )
+            return &default_fonts[n];
+
+    FIXME( "unhandled charset 0x%08x - use ANSI_CHARSET for default stock objects\n", csi.ciCharset );
+    return &default_fonts[0];
 }
 
 
@@ -604,7 +586,10 @@ static void init_gdi_shared(void)
     NtCurrentTeb()->Peb->GdiSharedHandleTable = gdi_shared;
 }
 
-HGDIOBJ get_stock_object( INT obj )
+/***********************************************************************
+ *           GetStockObject    (win32u.so)
+ */
+HGDIOBJ WINAPI GetStockObject( INT obj )
 {
     assert( obj >= 0 && obj <= STOCK_LAST + 1 && obj != 9 );
 
@@ -656,7 +641,7 @@ static void init_stock_objects( unsigned int dpi )
     create_font( &AnsiVarFont );
 
     /* language-dependent stock fonts */
-    deffonts = get_default_fonts(get_default_charset());
+    deffonts = get_default_fonts();
     create_font( &deffonts->SystemFont );
     create_font( &deffonts->DeviceDefaultFont );
 
@@ -1061,10 +1046,14 @@ static struct unix_funcs unix_funcs =
     NtGdiCreateDIBitmapInternal,
     NtGdiCreateMetafileDC,
     NtGdiDdDDICheckVidPnExclusiveOwnership,
+    NtGdiDdDDICloseAdapter,
     NtGdiDdDDICreateDCFromMemory,
     NtGdiDdDDIDestroyDCFromMemory,
     NtGdiDdDDIDestroyDevice,
     NtGdiDdDDIEscape,
+    NtGdiDdDDIOpenAdapterFromDeviceName,
+    NtGdiDdDDIOpenAdapterFromLuid,
+    NtGdiDdDDIQueryVideoMemoryInfo,
     NtGdiDdDDISetVidPnSourceOwner,
     NtGdiDeleteObjectApp,
     NtGdiDoPalette,
@@ -1150,56 +1139,128 @@ static struct unix_funcs unix_funcs =
     NtGdiUpdateColors,
     NtGdiWidenPath,
     NtUserActivateKeyboardLayout,
+    NtUserBeginPaint,
     NtUserCallHwnd,
     NtUserCallHwndParam,
     NtUserCallNextHookEx,
     NtUserCallNoParam,
     NtUserCallOneParam,
     NtUserCallTwoParam,
+    NtUserChangeClipboardChain,
     NtUserChangeDisplaySettings,
     NtUserClipCursor,
+    NtUserCloseClipboard,
     NtUserCountClipboardFormats,
+    NtUserCreateCaret,
+    NtUserCreateWindowEx,
+    NtUserDeferWindowPosAndBand,
     NtUserDestroyCursor,
+    NtUserDestroyMenu,
+    NtUserDestroyWindow,
+    NtUserDispatchMessage,
+    NtUserDragDetect,
+    NtUserDrawCaptionTemp,
     NtUserDrawIconEx,
+    NtUserDrawMenuBarTemp,
+    NtUserEmptyClipboard,
+    NtUserEnableMenuItem,
+    NtUserEndDeferWindowPosEx,
+    NtUserEndPaint,
     NtUserEnumDisplayDevices,
     NtUserEnumDisplayMonitors,
     NtUserEnumDisplaySettings,
+    NtUserExcludeUpdateRgn,
+    NtUserFlashWindowEx,
     NtUserGetAsyncKeyState,
+    NtUserGetClassInfoEx,
+    NtUserGetClipboardData,
     NtUserGetCursorInfo,
+    NtUserGetDCEx,
     NtUserGetDisplayConfigBufferSizes,
     NtUserGetIconInfo,
     NtUserGetKeyNameText,
     NtUserGetKeyboardLayoutList,
+    NtUserGetMenuBarInfo,
+    NtUserGetMessage,
     NtUserGetPriorityClipboardFormat,
     NtUserGetQueueStatus,
+    NtUserGetRawInputBuffer,
+    NtUserGetRawInputData,
+    NtUserGetSystemMenu,
+    NtUserGetUpdateRect,
+    NtUserGetUpdateRgn,
     NtUserGetUpdatedClipboardFormats,
+    NtUserGetWindowPlacement,
+    NtUserHideCaret,
+    NtUserHiliteMenuItem,
+    NtUserInternalGetWindowIcon,
+    NtUserInvalidateRect,
+    NtUserInvalidateRgn,
     NtUserIsClipboardFormatAvailable,
     NtUserMapVirtualKeyEx,
+    NtUserMessageCall,
+    NtUserMoveWindow,
+    NtUserMsgWaitForMultipleObjectsEx,
+    NtUserOpenClipboard,
+    NtUserPeekMessage,
+    NtUserPostMessage,
+    NtUserPostThreadMessage,
+    NtUserRedrawWindow,
+    NtUserRegisterClassExWOW,
     NtUserRegisterHotKey,
+    NtUserReleaseDC,
     NtUserScrollDC,
+    NtUserScrollWindowEx,
     NtUserSelectPalette,
+    NtUserSendInput,
+    NtUserSetActiveWindow,
+    NtUserSetCapture,
+    NtUserSetClipboardData,
+    NtUserSetClassLong,
+    NtUserSetClassLongPtr,
+    NtUserSetClassWord,
+    NtUserSetClipboardViewer,
     NtUserSetCursor,
     NtUserSetCursorIconData,
     NtUserSetCursorPos,
+    NtUserSetFocus,
+    NtUserSetInternalWindowPos,
+    NtUserSetLayeredWindowAttributes,
+    NtUserSetMenu,
+    NtUserSetParent,
     NtUserSetSysColors,
+    NtUserSetSystemMenu,
+    NtUserSetWindowLong,
+    NtUserSetWindowLongPtr,
+    NtUserSetWindowPlacement,
+    NtUserSetWindowPos,
+    NtUserSetWindowRgn,
+    NtUserSetWindowWord,
+    NtUserShowCaret,
     NtUserShowCursor,
+    NtUserShowWindow,
+    NtUserShowWindowAsync,
     NtUserSystemParametersInfo,
     NtUserSystemParametersInfoForDpi,
     NtUserToUnicodeEx,
+    NtUserTrackMouseEvent,
+    NtUserTrackPopupMenuEx,
+    NtUserTranslateAccelerator,
+    NtUserTranslateMessage,
+    NtUserUnregisterClass,
     NtUserUnregisterHotKey,
+    NtUserUpdateLayeredWindow,
     NtUserVkKeyScanEx,
+    NtUserWaitForInputIdle,
+    NtUserWindowFromPoint,
 
-    GetDCHook,
-    SetDCHook,
     SetDIBits,
-    SetHookFlags,
     __wine_get_brush_bitmap_info,
     __wine_get_file_outline_text_metric,
     __wine_get_icm_profile,
     __wine_get_vulkan_driver,
     __wine_get_wgl_driver,
-    __wine_set_display_driver,
-    __wine_set_visible_region,
+    __wine_send_input,
 };
 
 NTSTATUS gdi_init(void)
