@@ -51,11 +51,8 @@ static void * const syscalls[] =
     NtGdiCreateRectRgn,
     NtGdiCreateRoundRectRgn,
     NtGdiCreateSolidBrush,
-    NtGdiDdDDICloseAdapter,
     NtGdiDdDDICreateDevice,
-    NtGdiDdDDIOpenAdapterFromDeviceName,
     NtGdiDdDDIOpenAdapterFromHdc,
-    NtGdiDdDDIOpenAdapterFromLuid,
     NtGdiDdDDIQueryStatistics,
     NtGdiDdDDISetQueuedLimit,
     NtGdiDeleteClientObj,
@@ -70,7 +67,9 @@ static void * const syscalls[] =
     NtGdiGetBitmapBits,
     NtGdiGetBitmapDimension,
     NtGdiGetColorAdjustment,
+    NtGdiGetDCDword,
     NtGdiGetDCObject,
+    NtGdiGetDCPoint,
     NtGdiGetFontFileData,
     NtGdiGetFontFileInfo,
     NtGdiGetNearestPaletteIndex,
@@ -104,11 +103,25 @@ static void * const syscalls[] =
     NtUserAddClipboardFormatListener,
     NtUserAttachThreadInput,
     NtUserBuildHwndList,
+    NtUserCallMsgFilter,
+    NtUserCheckMenuItem,
+    NtUserChildWindowFromPointEx,
     NtUserCloseDesktop,
     NtUserCloseWindowStation,
+    NtUserCopyAcceleratorTable,
+    NtUserCreateAcceleratorTable,
     NtUserCreateDesktopEx,
     NtUserCreateWindowStation,
+    NtUserDeleteMenu,
+    NtUserDestroyAcceleratorTable,
+    NtUserEndMenu,
     NtUserFindExistingCursorIcon,
+    NtUserFindWindowEx,
+    NtUserGetAncestor,
+    NtUserGetAtomName,
+    NtUserGetCaretBlinkTime,
+    NtUserGetCaretPos,
+    NtUserGetClassName,
     NtUserGetClipboardFormatName,
     NtUserGetClipboardOwner,
     NtUserGetClipboardSequenceNumber,
@@ -125,6 +138,7 @@ static void * const syscalls[] =
     NtUserGetKeyboardLayoutName,
     NtUserGetKeyboardState,
     NtUserGetLayeredWindowAttributes,
+    NtUserGetMenuItemRect,
     NtUserGetMouseMovePointsEx,
     NtUserGetObjectInformation,
     NtUserGetOpenClipboardWindow,
@@ -133,22 +147,37 @@ static void * const syscalls[] =
     NtUserGetProp,
     NtUserGetSystemDpiForProcess,
     NtUserGetThreadDesktop,
+    NtUserGetTitleBarInfo,
+    NtUserGetWindowRgnEx,
+    NtUserInitializeClientPfnArrays,
+    NtUserInternalGetWindowText,
+    NtUserKillTimer,
+    NtUserLockWindowUpdate,
+    NtUserMenuItemFromPoint,
     NtUserNotifyWinEvent,
     NtUserOpenDesktop,
     NtUserOpenInputDesktop,
     NtUserOpenWindowStation,
     NtUserRemoveClipboardFormatListener,
+    NtUserRemoveMenu,
     NtUserRemoveProp,
     NtUserSetKeyboardState,
+    NtUserSetMenuContextHelpId,
+    NtUserSetMenuDefaultItem,
     NtUserSetObjectInformation,
     NtUserSetProcessDpiAwarenessContext,
     NtUserSetProcessWindowStation,
     NtUserSetProp,
+    NtUserSetSystemTimer,
     NtUserSetThreadDesktop,
+    NtUserSetTimer,
     NtUserSetWinEventHook,
     NtUserSetWindowsHookEx,
+    NtUserThunkedMenuInfo,
+    NtUserThunkedMenuItemInfo,
     NtUserUnhookWinEvent,
     NtUserUnhookWindowsHookEx,
+    NtUserWindowFromDC,
 };
 
 static BYTE arguments[ARRAY_SIZE(syscalls)];
