@@ -149,6 +149,12 @@ INT WINAPI NtGdiEndPage( HDC hdc )
     return unix_funcs->pNtGdiEndPage( hdc );
 }
 
+BOOL WINAPI NtUserEndPaint( HWND hwnd, const PAINTSTRUCT *ps )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserEndPaint( hwnd, ps );
+}
+
 BOOL WINAPI NtGdiEnumFonts( HDC hdc, ULONG type, ULONG win32_compat, ULONG face_name_len,
                             const WCHAR *face_name, ULONG charset, ULONG *count, void *buf )
 {
@@ -665,6 +671,12 @@ NTSTATUS WINAPI NtGdiDdDDICheckVidPnExclusiveOwnership( const D3DKMT_CHECKVIDPNE
     return unix_funcs->pNtGdiDdDDICheckVidPnExclusiveOwnership( desc );
 }
 
+NTSTATUS WINAPI NtGdiDdDDICloseAdapter( const D3DKMT_CLOSEADAPTER *desc )
+{
+    if (!unix_funcs) return STATUS_NOT_SUPPORTED;
+    return unix_funcs->pNtGdiDdDDICloseAdapter( desc );
+}
+
 NTSTATUS WINAPI NtGdiDdDDICreateDCFromMemory( D3DKMT_CREATEDCFROMMEMORY *desc )
 {
     if (!unix_funcs) return STATUS_NOT_SUPPORTED;
@@ -689,6 +701,24 @@ NTSTATUS WINAPI NtGdiDdDDIEscape( const D3DKMT_ESCAPE *desc )
     return unix_funcs->pNtGdiDdDDIEscape( desc );
 }
 
+NTSTATUS WINAPI NtGdiDdDDIOpenAdapterFromDeviceName( D3DKMT_OPENADAPTERFROMDEVICENAME *desc )
+{
+    if (!unix_funcs) return STATUS_NOT_SUPPORTED;
+    return unix_funcs->pNtGdiDdDDIOpenAdapterFromDeviceName( desc );
+}
+
+NTSTATUS WINAPI NtGdiDdDDIOpenAdapterFromLuid( D3DKMT_OPENADAPTERFROMLUID *desc )
+{
+    if (!unix_funcs) return STATUS_NOT_SUPPORTED;
+    return unix_funcs->pNtGdiDdDDIOpenAdapterFromLuid( desc );
+}
+
+NTSTATUS WINAPI NtGdiDdDDIQueryVideoMemoryInfo( D3DKMT_QUERYVIDEOMEMORYINFO *desc )
+{
+    if (!unix_funcs) return STATUS_NOT_SUPPORTED;
+    return unix_funcs->pNtGdiDdDDIQueryVideoMemoryInfo( desc );
+}
+
 NTSTATUS WINAPI NtGdiDdDDISetVidPnSourceOwner( const D3DKMT_SETVIDPNSOURCEOWNER *desc )
 {
     if (!unix_funcs) return STATUS_NOT_SUPPORTED;
@@ -701,6 +731,12 @@ HKL WINAPI NtUserActivateKeyboardLayout( HKL layout, UINT flags )
     return unix_funcs->pNtUserActivateKeyboardLayout( layout, flags );
 }
 
+HDC WINAPI NtUserBeginPaint( HWND hwnd, PAINTSTRUCT *ps )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserBeginPaint( hwnd, ps );
+}
+
 LRESULT WINAPI NtUserCallNextHookEx( HHOOK hhook, INT code, WPARAM wparam, LPARAM lparam )
 {
     if (!unix_funcs) return 0;
@@ -725,18 +761,30 @@ ULONG_PTR WINAPI NtUserCallTwoParam( ULONG_PTR arg1, ULONG_PTR arg2, ULONG code
     return unix_funcs->pNtUserCallTwoParam( arg1, arg2, code );
 }
 
-DWORD WINAPI NtUserCallHwnd( HWND hwnd, DWORD code )
+ULONG_PTR WINAPI NtUserCallHwnd( HWND hwnd, DWORD code )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserCallHwnd( hwnd, code );
 }
 
-DWORD WINAPI NtUserCallHwndParam( HWND hwnd, DWORD_PTR param, DWORD code )
+ULONG_PTR WINAPI NtUserCallHwndParam( HWND hwnd, DWORD_PTR param, DWORD code )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserCallHwndParam( hwnd, param, code );
 }
 
+BOOL WINAPI NtUserCloseClipboard(void)
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserCloseClipboard();
+}
+
+BOOL WINAPI NtUserChangeClipboardChain( HWND hwnd, HWND next )
+{
+    if (!unix_funcs) return DISP_CHANGE_FAILED;
+    return unix_funcs->pNtUserChangeClipboardChain( hwnd, next );
+}
+
 LONG WINAPI NtUserChangeDisplaySettings( UNICODE_STRING *devname, DEVMODEW *devmode, HWND hwnd,
                                          DWORD flags, void *lparam )
 {
@@ -756,12 +804,70 @@ INT WINAPI NtUserCountClipboardFormats(void)
     return unix_funcs->pNtUserCountClipboardFormats();
 }
 
+BOOL WINAPI NtUserCreateCaret( HWND hwnd, HBITMAP bitmap, int width, int height )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserCreateCaret( hwnd, bitmap, width, height );
+}
+
+HWND WINAPI NtUserCreateWindowEx( DWORD ex_style, UNICODE_STRING *class_name,
+                                  UNICODE_STRING *version, UNICODE_STRING *window_name,
+                                  DWORD style, INT x, INT y, INT width, INT height,
+                                  HWND parent, HMENU menu, HINSTANCE instance, void *params,
+                                  DWORD flags, CBT_CREATEWNDW *cbtc, DWORD unk, BOOL ansi )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserCreateWindowEx( ex_style, class_name, version, window_name,
+                                              style, x, y, width, height, parent, menu,
+                                              instance, params, flags, cbtc, unk, ansi );
+}
+
+HDWP WINAPI NtUserDeferWindowPosAndBand( HDWP hdwp, HWND hwnd, HWND after,
+                                         INT x, INT y, INT cx, INT cy,
+                                         UINT flags, UINT unk1, UINT unk2 )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserDeferWindowPosAndBand( hdwp, hwnd, after, x, y, cx, cy,
+                                                     flags, unk1, unk2 );
+}
+
 BOOL WINAPI NtUserDestroyCursor( HCURSOR cursor, ULONG arg )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->pNtUserDestroyCursor( cursor, arg );
 }
 
+BOOL WINAPI NtUserDestroyMenu( HMENU handle )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserDestroyMenu( handle );
+}
+
+BOOL WINAPI NtUserDestroyWindow( HWND hwnd )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserDestroyWindow( hwnd );
+}
+
+LRESULT WINAPI NtUserDispatchMessage( const MSG *msg )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserDispatchMessage( msg );
+}
+
+BOOL WINAPI NtUserDragDetect( HWND hwnd, int x, int y )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserDragDetect( hwnd, x, y );
+}
+
+BOOL WINAPI NtUserDrawCaptionTemp( HWND hwnd, HDC hdc, const RECT *rect, HFONT font,
+                                   HICON icon, const WCHAR *str, UINT flags )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserDrawCaptionTemp( hwnd, hdc, rect, font, icon, str, flags );
+}
+
 BOOL WINAPI NtUserDrawIconEx( HDC hdc, INT x0, INT y0, HICON icon, INT width,
                               INT height, UINT istep, HBRUSH hbr, UINT flags )
 {
@@ -769,6 +875,30 @@ BOOL WINAPI NtUserDrawIconEx( HDC hdc, INT x0, INT y0, HICON icon, INT width,
     return unix_funcs->pNtUserDrawIconEx( hdc, x0, y0, icon, width, height, istep, hbr, flags );
 }
 
+DWORD WINAPI NtUserDrawMenuBarTemp( HWND hwnd, HDC hdc, RECT *rect, HMENU handle, HFONT font )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserDrawMenuBarTemp( hwnd, hdc, rect, handle, font );
+}
+
+BOOL WINAPI NtUserEnableMenuItem( HMENU handle, UINT id, UINT flags )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserEnableMenuItem( handle, id, flags );
+}
+
+BOOL WINAPI NtUserEndDeferWindowPosEx( HDWP hdwp, BOOL async )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserEndDeferWindowPosEx( hdwp, async );
+}
+
+BOOL WINAPI NtUserEmptyClipboard(void)
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserEmptyClipboard();
+}
+
 NTSTATUS WINAPI NtUserEnumDisplayDevices( UNICODE_STRING *device, DWORD index,
                                           DISPLAY_DEVICEW *info, DWORD flags )
 {
@@ -789,18 +919,49 @@ BOOL WINAPI NtUserEnumDisplaySettings( UNICODE_STRING *device, DWORD mode,
     return unix_funcs->pNtUserEnumDisplaySettings( device, mode, dev_mode, flags );
 }
 
+INT WINAPI NtUserExcludeUpdateRgn( HDC hdc, HWND hwnd )
+{
+    if (!unix_funcs) return ERROR;
+    return unix_funcs->pNtUserExcludeUpdateRgn( hdc, hwnd );
+}
+
+BOOL WINAPI NtUserFlashWindowEx( FLASHWINFO *info )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserFlashWindowEx( info );
+}
+
 SHORT WINAPI NtUserGetAsyncKeyState( INT key )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserGetAsyncKeyState( key );
 }
 
+ATOM WINAPI NtUserGetClassInfoEx( HINSTANCE instance, UNICODE_STRING *name, WNDCLASSEXW *wc,
+                                  struct client_menu_name *menu_name, BOOL ansi )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserGetClassInfoEx( instance, name, wc, menu_name, ansi );
+}
+
+HANDLE WINAPI NtUserGetClipboardData( UINT format, struct get_clipboard_params *params )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserGetClipboardData( format, params );
+}
+
 BOOL WINAPI NtUserGetCursorInfo( CURSORINFO *info )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->pNtUserGetCursorInfo( info );
 }
 
+HDC WINAPI NtUserGetDCEx( HWND hwnd, HRGN clip_rgn, DWORD flags )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserGetDCEx( hwnd, clip_rgn, flags );
+}
+
 LONG WINAPI NtUserGetDisplayConfigBufferSizes( UINT32 flags, UINT32 *num_path_info,
                                                UINT32 *num_mode_info )
 {
@@ -827,6 +988,54 @@ INT WINAPI NtUserGetKeyNameText( LONG lparam, WCHAR *buffer, INT size )
     return unix_funcs->pNtUserGetKeyNameText( lparam, buffer, size );
 }
 
+BOOL WINAPI NtUserGetMenuBarInfo( HWND hwnd, LONG id, LONG item, MENUBARINFO *info )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserGetMenuBarInfo( hwnd, id, item, info );
+}
+
+BOOL WINAPI NtUserGetMessage( MSG *msg, HWND hwnd, UINT first, UINT last )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserGetMessage( msg, hwnd, first, last );
+}
+
+HMENU WINAPI NtUserGetSystemMenu( HWND hwnd, BOOL revert )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserGetSystemMenu( hwnd, revert );
+}
+
+BOOL WINAPI NtUserGetUpdateRect( HWND hwnd, RECT *rect, BOOL erase )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserGetUpdateRect( hwnd, rect, erase );
+}
+
+INT WINAPI NtUserGetUpdateRgn( HWND hwnd, HRGN hrgn, BOOL erase )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserGetUpdateRgn( hwnd, hrgn, erase );
+}
+
+BOOL WINAPI NtUserHideCaret( HWND hwnd )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserHideCaret( hwnd );
+}
+
+BOOL WINAPI NtUserHiliteMenuItem( HWND hwnd, HMENU handle, UINT item, UINT hilite )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserHiliteMenuItem( hwnd, handle, item, hilite );
+}
+
+BOOL WINAPI NtUserMoveWindow( HWND hwnd, INT x, INT y, INT cx, INT cy, BOOL repaint )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserMoveWindow( hwnd, x, y, cx, cy, repaint );
+}
+
 INT WINAPI NtUserGetPriorityClipboardFormat( UINT *list, INT count )
 {
     if (!unix_funcs) return 0;
@@ -839,12 +1048,49 @@ DWORD WINAPI NtUserGetQueueStatus( UINT flags )
     return unix_funcs->pNtUserGetQueueStatus( flags );
 }
 
+UINT WINAPI DECLSPEC_HOTPATCH NtUserGetRawInputBuffer( RAWINPUT *data, UINT *data_size, UINT header_size )
+{
+    if (!unix_funcs) return ~0u;
+    return unix_funcs->pNtUserGetRawInputBuffer( data, data_size, header_size );
+}
+
+UINT WINAPI NtUserGetRawInputData( HRAWINPUT rawinput, UINT command, void *data, UINT *data_size, UINT header_size )
+{
+    if (!unix_funcs) return ~0u;
+    return unix_funcs->pNtUserGetRawInputData( rawinput, command, data, data_size, header_size );
+}
+
 BOOL WINAPI NtUserGetUpdatedClipboardFormats( UINT *formats, UINT size, UINT *out_size )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->pNtUserGetUpdatedClipboardFormats( formats, size, out_size );
 }
 
+
+BOOL WINAPI NtUserGetWindowPlacement( HWND hwnd, WINDOWPLACEMENT *placement )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserGetWindowPlacement( hwnd, placement );
+}
+
+HICON WINAPI NtUserInternalGetWindowIcon( HWND hwnd, UINT type )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserInternalGetWindowIcon( hwnd, type );
+}
+
+BOOL WINAPI NtUserInvalidateRect( HWND hwnd, const RECT *rect, BOOL erase )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserInvalidateRect( hwnd, rect, erase );
+}
+
+BOOL WINAPI NtUserInvalidateRgn( HWND hwnd, HRGN hrgn, BOOL erase )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserInvalidateRgn( hwnd, hrgn, erase );
+}
+
 BOOL WINAPI NtUserIsClipboardFormatAvailable( UINT format )
 {
     if (!unix_funcs) return FALSE;
@@ -857,12 +1103,70 @@ UINT WINAPI NtUserMapVirtualKeyEx( UINT code, UINT type, HKL layout )
     return unix_funcs->pNtUserMapVirtualKeyEx( code, type, layout );
 }
 
+LRESULT WINAPI NtUserMessageCall( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam,
+                                  void *result_info, DWORD type, BOOL ansi )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserMessageCall( hwnd, msg, wparam, lparam, result_info, type, ansi );
+}
+
+DWORD WINAPI NtUserMsgWaitForMultipleObjectsEx( DWORD count, const HANDLE *handles,
+                                                DWORD timeout, DWORD mask, DWORD flags )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserMsgWaitForMultipleObjectsEx( count, handles, timeout, mask, flags );
+}
+
+BOOL WINAPI NtUserOpenClipboard( HWND hwnd, ULONG unk )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserOpenClipboard( hwnd, unk );
+}
+
+BOOL WINAPI NtUserPeekMessage( MSG *msg_out, HWND hwnd, UINT first, UINT last, UINT flags )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserPeekMessage( msg_out, hwnd, first, last, flags );
+}
+
+BOOL WINAPI NtUserPostMessage( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserPostMessage( hwnd, msg, wparam, lparam );
+}
+
+BOOL WINAPI NtUserPostThreadMessage( DWORD thread, UINT msg, WPARAM wparam, LPARAM lparam )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserPostThreadMessage( thread, msg, wparam, lparam );
+}
+
+BOOL WINAPI NtUserRedrawWindow( HWND hwnd, const RECT *rect, HRGN hrgn, UINT flags )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserRedrawWindow( hwnd, rect, hrgn, flags );
+}
+
+ATOM WINAPI NtUserRegisterClassExWOW( const WNDCLASSEXW *wc, UNICODE_STRING *name, UNICODE_STRING *version,
+                                      struct client_menu_name *client_menu_name, DWORD fnid, DWORD flags,
+                                      DWORD *wow )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserRegisterClassExWOW( wc, name, version, client_menu_name, fnid, flags, wow );
+}
+
 BOOL WINAPI NtUserRegisterHotKey( HWND hwnd, INT id, UINT modifiers, UINT vk )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->pNtUserRegisterHotKey( hwnd, id, modifiers, vk );
 }
 
+INT WINAPI NtUserReleaseDC( HWND hwnd, HDC hdc )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserReleaseDC( hwnd, hdc );
+}
+
 BOOL WINAPI NtUserScrollDC( HDC hdc, INT dx, INT dy, const RECT *scroll, const RECT *clip,
                             HRGN ret_update_rgn, RECT *update_rect )
 {
@@ -870,18 +1174,75 @@ BOOL WINAPI NtUserScrollDC( HDC hdc, INT dx, INT dy, const RECT *scroll, const R
     return unix_funcs->pNtUserScrollDC( hdc, dx, dy, scroll, clip, ret_update_rgn, update_rect );
 }
 
+INT WINAPI NtUserScrollWindowEx( HWND hwnd, INT dx, INT dy, const RECT *rect,
+                                 const RECT *clip_rect, HRGN update_rgn,
+                                 RECT *update_rect, UINT flags )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserScrollWindowEx( hwnd, dx, dy, rect, clip_rect,
+                                              update_rgn, update_rect, flags );
+}
+
 HPALETTE WINAPI NtUserSelectPalette( HDC hdc, HPALETTE hpal, WORD bkg )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserSelectPalette( hdc, hpal, bkg );
 }
 
+UINT WINAPI NtUserSendInput( UINT count, INPUT *inputs, int size )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSendInput( count, inputs, size );
+}
+
+HWND WINAPI NtUserSetActiveWindow( HWND hwnd )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetActiveWindow( hwnd );
+}
+
+HWND WINAPI NtUserSetCapture( HWND hwnd )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetCapture( hwnd );
+}
+
+NTSTATUS WINAPI NtUserSetClipboardData( UINT format, HANDLE handle, struct set_clipboard_params *params )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetClipboardData( format, handle, params );
+}
+
 HCURSOR WINAPI NtUserSetCursor( HCURSOR cursor )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserSetCursor( cursor );
 }
 
+DWORD WINAPI NtUserSetClassLong( HWND hwnd, INT offset, LONG newval, BOOL ansi )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetClassLong( hwnd, offset, newval, ansi );
+}
+
+ULONG_PTR WINAPI NtUserSetClassLongPtr( HWND hwnd, INT offset, LONG_PTR newval, BOOL ansi )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetClassLongPtr( hwnd, offset, newval, ansi );
+}
+
+WORD WINAPI NtUserSetClassWord( HWND hwnd, INT offset, WORD newval )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetClassWord( hwnd, offset, newval );
+}
+
+HWND WINAPI NtUserSetClipboardViewer( HWND hwnd )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetClipboardViewer( hwnd );
+}
+
 BOOL WINAPI NtUserSetCursorIconData( HCURSOR cursor, UNICODE_STRING *module, UNICODE_STRING *res_name,
                                      struct cursoricon_desc *desc )
 {
@@ -895,18 +1256,108 @@ BOOL WINAPI NtUserSetCursorPos( INT x, INT y )
     return unix_funcs->pNtUserSetCursorPos( x, y );
 }
 
+HWND WINAPI NtUserSetFocus( HWND hwnd )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserSetFocus( hwnd );
+}
+
+void WINAPI NtUserSetInternalWindowPos( HWND hwnd, UINT cmd, RECT *rect, POINT *pt )
+{
+    if (!unix_funcs) return;
+    return unix_funcs->pNtUserSetInternalWindowPos( hwnd, cmd, rect, pt );
+}
+
+BOOL WINAPI NtUserSetLayeredWindowAttributes( HWND hwnd, COLORREF key, BYTE alpha, DWORD flags )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserSetLayeredWindowAttributes( hwnd, key, alpha, flags );
+}
+
+BOOL WINAPI NtUserSetMenu( HWND hwnd, HMENU menu )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserSetMenu( hwnd, menu );
+}
+
+HWND WINAPI NtUserSetParent( HWND hwnd, HWND parent )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetParent( hwnd, parent );
+}
+
 BOOL WINAPI NtUserSetSysColors( INT count, const INT *colors, const COLORREF *values )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->pNtUserSetSysColors( count, colors, values );
 }
 
+BOOL WINAPI NtUserSetSystemMenu( HWND hwnd, HMENU menu )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserSetSystemMenu( hwnd, menu );
+}
+
+LONG WINAPI NtUserSetWindowLong( HWND hwnd, INT offset, LONG newval, BOOL ansi )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetWindowLong( hwnd, offset, newval, ansi );
+}
+
+LONG_PTR WINAPI NtUserSetWindowLongPtr( HWND hwnd, INT offset, LONG_PTR newval, BOOL ansi )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetWindowLongPtr( hwnd, offset, newval, ansi );
+}
+
+BOOL WINAPI NtUserSetWindowPlacement( HWND hwnd, const WINDOWPLACEMENT *wpl )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetWindowPlacement( hwnd, wpl );
+}
+
+BOOL WINAPI NtUserSetWindowPos( HWND hwnd, HWND after, INT x, INT y, INT cx, INT cy, UINT flags )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetWindowPos( hwnd, after, x, y, cx, cy, flags );
+}
+
+int WINAPI NtUserSetWindowRgn( HWND hwnd, HRGN hrgn, BOOL redraw )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetWindowRgn( hwnd, hrgn, redraw );
+}
+
+WORD WINAPI NtUserSetWindowWord( HWND hwnd, INT offset, WORD newval )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserSetWindowWord( hwnd, offset, newval );
+}
+
+BOOL WINAPI NtUserShowCaret( HWND hwnd )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserShowCaret( hwnd );
+}
+
 INT WINAPI NtUserShowCursor( BOOL show )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserShowCursor( show );
 }
 
+BOOL WINAPI NtUserShowWindowAsync( HWND hwnd, INT cmd )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserShowWindowAsync( hwnd, cmd );
+}
+
+BOOL WINAPI NtUserShowWindow( HWND hwnd, INT cmd )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserShowWindow( hwnd, cmd );
+}
+
 BOOL WINAPI NtUserSystemParametersInfo( UINT action, UINT val, PVOID ptr, UINT winini )
 {
     if (!unix_funcs) return FALSE;
@@ -926,28 +1377,69 @@ INT WINAPI NtUserToUnicodeEx( UINT virt, UINT scan, const BYTE *state,
     return unix_funcs->pNtUserToUnicodeEx( virt, scan, state, str, size, flags, layout );
 }
 
+BOOL WINAPI NtUserTrackMouseEvent( TRACKMOUSEEVENT *info )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserTrackMouseEvent( info );
+}
+
+BOOL WINAPI NtUserTrackPopupMenuEx( HMENU handle, UINT flags, INT x, INT y, HWND hwnd,
+                                    TPMPARAMS *params )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserTrackPopupMenuEx( handle, flags, x, y, hwnd, params );
+}
+
+INT WINAPI NtUserTranslateAccelerator( HWND hwnd, HACCEL accel, MSG *msg )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserTranslateAccelerator( hwnd, accel, msg );
+}
+
+BOOL WINAPI NtUserTranslateMessage( const MSG *msg, UINT flags )
+{
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserTranslateMessage( msg, flags );
+}
+
+BOOL WINAPI NtUserUnregisterClass( UNICODE_STRING *name, HINSTANCE instance,
+                                   struct client_menu_name *client_menu_name )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserUnregisterClass( name, instance, client_menu_name );
+}
+
 BOOL WINAPI NtUserUnregisterHotKey( HWND hwnd, INT id )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->pNtUserUnregisterHotKey( hwnd, id );
 }
 
+BOOL WINAPI NtUserUpdateLayeredWindow( HWND hwnd, HDC hdc_dst, const POINT *pts_dst, const SIZE *size,
+                                       HDC hdc_src, const POINT *pts_src, COLORREF key,
+                                       const BLENDFUNCTION *blend, DWORD flags, const RECT *dirty )
+{
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->pNtUserUpdateLayeredWindow( hwnd, hdc_dst, pts_dst, size, hdc_src, pts_src,
+                                                   key, blend, flags, dirty );
+}
+
 WORD WINAPI NtUserVkKeyScanEx( WCHAR chr, HKL layout )
 {
     if (!unix_funcs) return 0;
     return unix_funcs->pNtUserVkKeyScanEx( chr, layout );
 }
 
-DWORD_PTR WINAPI GetDCHook( HDC hdc, DCHOOKPROC *proc )
+DWORD WINAPI NtUserWaitForInputIdle( HANDLE process, DWORD timeout, BOOL wow )
 {
     if (!unix_funcs) return 0;
-    return unix_funcs->pGetDCHook( hdc, proc );
+    return unix_funcs->pNtUserWaitForInputIdle( process, timeout, wow );
 }
 
-BOOL WINAPI SetDCHook( HDC hdc, DCHOOKPROC proc, DWORD_PTR data )
+HWND WINAPI NtUserWindowFromPoint( LONG x, LONG y )
 {
-    if (!unix_funcs) return FALSE;
-    return unix_funcs->pSetDCHook( hdc, proc, data );
+    if (!unix_funcs) return 0;
+    return unix_funcs->pNtUserWindowFromPoint( x, y );
 }
 
 INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
@@ -958,25 +1450,12 @@ INT WINAPI SetDIBits( HDC hdc, HBITMAP hbitmap, UINT startscan,
     return unix_funcs->pSetDIBits( hdc, hbitmap, startscan, lines, bits, info, coloruse );
 }
 
-WORD WINAPI SetHookFlags( HDC hdc, WORD flags )
-{
-    if (!unix_funcs) return 0;
-    return unix_funcs->pSetHookFlags( hdc, flags );
-}
-
 BOOL CDECL __wine_get_icm_profile( HDC hdc, BOOL allow_default, DWORD *size, WCHAR *filename )
 {
     if (!unix_funcs) return FALSE;
     return unix_funcs->get_icm_profile( hdc, allow_default, size, filename );
 }
 
-void CDECL __wine_set_visible_region( HDC hdc, HRGN hrgn, const RECT *vis_rect, const RECT *device_rect,
-                                      struct window_surface *surface )
-{
-    if (!unix_funcs) return;
-    unix_funcs->set_visible_region( hdc, hrgn, vis_rect, device_rect, surface );
-}
-
 BOOL CDECL __wine_get_brush_bitmap_info( HBRUSH handle, BITMAPINFO *info, void *bits, UINT *usage )
 {
     if (!unix_funcs) return FALSE;
@@ -1001,13 +1480,10 @@ struct opengl_funcs * CDECL __wine_get_wgl_driver( HDC hdc, UINT version )
     return unix_funcs->get_wgl_driver( hdc, version );
 }
 
-/***********************************************************************
- *           __wine_set_display_driver    (win32u.@)
- */
-void CDECL __wine_set_display_driver( struct user_driver_funcs *funcs, UINT version )
+BOOL CDECL __wine_send_input( HWND hwnd, const INPUT *input, const RAWINPUT *rawinput )
 {
-    if (!unix_funcs) return;
-    return unix_funcs->set_display_driver( funcs, version );
+    if (!unix_funcs) return FALSE;
+    return unix_funcs->wine_send_input( hwnd, input, rawinput );
 }
 
 extern void wrappers_init( unixlib_handle_t handle )
