@@ -103,6 +103,7 @@ struct midi_dest
 
     MIDIOUTCAPSW caps;
     MIDIOPENDESC midiDesc;
+    BYTE runningStatus;
     WORD wFlags;
 };
 
@@ -116,7 +117,7 @@ struct midi_src
     MIDIOPENDESC midiDesc;
     LPMIDIHDR lpQueueHdr;
     WORD wFlags;
-    DWORD startTime;
+    UINT startTime;
 };
 
 static MIDIClientRef midi_client;
@@ -143,7 +144,7 @@ static void midi_in_lock(BOOL lock)
 }
 
 static void set_in_notify(struct notify_context *notify, struct midi_src *src, WORD dev_id, WORD msg,
-                          DWORD_PTR param_1, DWORD_PTR param_2)
+                          UINT_PTR param_1, UINT_PTR param_2)
 {
     notify->send_notify = TRUE;
     notify->dev_id = dev_id;
@@ -220,7 +221,7 @@ static uint64_t get_time_ms(void)
 static void process_sysex_packet(struct midi_src *src, MIDIPacket *packet)
 {
     unsigned int pos = 0, len = packet->length, copy_len;
-    DWORD current_time = get_time_ms() - src->startTime;
+    UINT current_time = get_time_ms() - src->startTime;
     struct notify_context notify;
 
     src->state |= 2;
@@ -244,7 +245,7 @@ static void process_sysex_packet(struct midi_src *src, MIDIPacket *packet)
             src->lpQueueHdr = hdr->lpNext;
             hdr->dwFlags &= ~MHDR_INQUEUE;
             hdr->dwFlags |= MHDR_DONE;
-            set_in_notify(&notify, src, src->wDevID, MIM_LONGDATA, (DWORD_PTR)hdr, current_time);
+            set_in_notify(&notify, src, src->wDevID, MIM_LONGDATA, (UINT_PTR)hdr, current_time);
             notify_post(&notify);
             src->state &= ~2;
         }
@@ -255,7 +256,7 @@ static void process_sysex_packet(struct midi_src *src, MIDIPacket *packet)
 
 static void process_small_packet(struct midi_src *src, MIDIPacket *packet)
 {
-    DWORD current_time = get_time_ms() - src->startTime, data;
+    UINT current_time = get_time_ms() - src->startTime, data;
     struct notify_context notify;
     unsigned int pos = 0;
 
@@ -547,7 +548,7 @@ static BOOL synth_unit_close(AUGraph graph)
 }
 
 static void set_out_notify(struct notify_context *notify, struct midi_dest *dest, WORD dev_id, WORD msg,
-                           DWORD_PTR param_1, DWORD_PTR param_2)
+                           UINT_PTR param_1, UINT_PTR param_2)
 {
     notify->send_notify = TRUE;
     notify->dev_id = dev_id;
@@ -560,7 +561,7 @@ static void set_out_notify(struct notify_context *notify, struct midi_dest *dest
     notify->instance = dest->midiDesc.dwInstance;
 }
 
-static DWORD midi_out_open(WORD dev_id, MIDIOPENDESC *midi_desc, DWORD flags, struct notify_context *notify)
+static UINT midi_out_open(WORD dev_id, MIDIOPENDESC *midi_desc, UINT flags, struct notify_context *notify)
 {
     struct midi_dest *dest;
 
@@ -598,6 +599,7 @@ static DWORD midi_out_open(WORD dev_id, MIDIOPENDESC *midi_desc, DWORD flags, st
             return MMSYSERR_ERROR;
         }
     }
+    dest->runningStatus = 0;
     dest->wFlags = HIWORD(flags & CALLBACK_TYPEMASK);
     dest->midiDesc = *midi_desc;
 
@@ -606,7 +608,7 @@ static DWORD midi_out_open(WORD dev_id, MIDIOPENDESC *midi_desc, DWORD flags, st
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_out_close(WORD dev_id, struct notify_context *notify)
+static UINT midi_out_close(WORD dev_id, struct notify_context *notify)
 {
     struct midi_dest *dest;
 
@@ -642,7 +644,7 @@ static void midi_send(MIDIPortRef port, MIDIEndpointRef dest, UInt8 *buffer, uns
     if (packet) MIDISend(port, dest, packet_list);
 }
 
-static DWORD midi_out_data(WORD dev_id, DWORD data)
+static UINT midi_out_data(WORD dev_id, UINT data)
 {
     struct midi_dest *dest;
     UInt8 bytes[3];
@@ -655,12 +657,30 @@ static DWORD midi_out_data(WORD dev_id, DWORD data)
         WARN("bad device ID : %d\n", dev_id);
         return MMSYSERR_BADDEVICEID;
     }
+    dest = dests + dev_id;
 
     bytes[0] = data & 0xff;
-    bytes[1] = (data >> 8) & 0xff;
-    bytes[2] = (data >> 16) & 0xff;
+    if (bytes[0] & 0x80)
+    {
+        bytes[1] = (data >> 8) & 0xff;
+        bytes[2] = (data >> 16) & 0xff;
+        if (bytes[0] < 0xF0)
+            dest->runningStatus = bytes[0];
+        else if (bytes[0] <= 0xF7)
+            dest->runningStatus = 0;
+    }
+    else if (dest->runningStatus)
+    {
+        bytes[0] = dest->runningStatus;
+        bytes[1] = data & 0xff;
+        bytes[2] = (data >> 8) & 0xff;
+    }
+    else
+    {
+        FIXME("ooch %x\n", data);
+        return MMSYSERR_NOERROR;
+    }
 
-    dest = dests + dev_id;
     if (dest->caps.wTechnology == MOD_SYNTH)
     {
         sc = MusicDeviceMIDIEvent(dest->synth, bytes[0], bytes[1], bytes[2], 0);
@@ -678,7 +698,7 @@ static DWORD midi_out_data(WORD dev_id, DWORD data)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_out_long_data(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size, struct notify_context *notify)
+static UINT midi_out_long_data(WORD dev_id, MIDIHDR *hdr, UINT hdr_size, struct notify_context *notify)
 {
     struct midi_dest *dest;
     OSStatus sc;
@@ -726,15 +746,16 @@ static DWORD midi_out_long_data(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size, struc
     else if (dest->caps.wTechnology == MOD_MIDIPORT)
         midi_send(midi_out_port, dest->dest, (UInt8 *)hdr->lpData, hdr->dwBufferLength);
 
+    dest->runningStatus = 0;
     hdr->dwFlags &= ~MHDR_INQUEUE;
     hdr->dwFlags |= MHDR_DONE;
 
-    set_out_notify(notify, dest, dev_id, MOM_DONE, (DWORD_PTR)hdr, 0);
+    set_out_notify(notify, dest, dev_id, MOM_DONE, (UINT_PTR)hdr, 0);
 
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_out_prepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
+static UINT midi_out_prepare(WORD dev_id, MIDIHDR *hdr, UINT hdr_size)
 {
     TRACE("dev_id = %d midi_hdr = %p hdr_size = %d\n", dev_id, hdr, hdr_size);
 
@@ -749,7 +770,7 @@ static DWORD midi_out_prepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_out_unprepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
+static UINT midi_out_unprepare(WORD dev_id, MIDIHDR *hdr, UINT hdr_size)
 {
     TRACE("dev_id = %d midi_hdr = %p hdr_size = %d\n", dev_id, hdr, hdr_size);
 
@@ -764,7 +785,7 @@ static DWORD midi_out_unprepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_out_get_devcaps(WORD dev_id, MIDIOUTCAPSW *caps, DWORD size)
+static UINT midi_out_get_devcaps(WORD dev_id, MIDIOUTCAPSW *caps, UINT size)
 {
     TRACE("dev_id = %d caps = %p size = %d\n", dev_id, caps, size);
 
@@ -783,13 +804,13 @@ static DWORD midi_out_get_devcaps(WORD dev_id, MIDIOUTCAPSW *caps, DWORD size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_out_get_num_devs(void)
+static UINT midi_out_get_num_devs(void)
 {
     TRACE("\n");
     return num_dests;
 }
 
-static DWORD midi_out_get_volume(WORD dev_id, DWORD *volume)
+static UINT midi_out_get_volume(WORD dev_id, UINT *volume)
 {
     TRACE("%d\n", dev_id);
 
@@ -818,7 +839,7 @@ static DWORD midi_out_get_volume(WORD dev_id, DWORD *volume)
     return MMSYSERR_NOTSUPPORTED;
 }
 
-static DWORD midi_out_set_volume(WORD dev_id, DWORD volume)
+static UINT midi_out_set_volume(WORD dev_id, UINT volume)
 {
     TRACE("dev_id = %d vol = %08x\n", dev_id, volume);
 
@@ -843,7 +864,7 @@ static DWORD midi_out_set_volume(WORD dev_id, DWORD volume)
     return MMSYSERR_NOTSUPPORTED;
 }
 
-static DWORD midi_out_reset(WORD dev_id)
+static UINT midi_out_reset(WORD dev_id)
 {
     unsigned chn;
 
@@ -866,11 +887,13 @@ static DWORD midi_out_reset(WORD dev_id)
     }
     else FIXME("MOD_MIDIPORT\n");
 
+    dests[dev_id].runningStatus = 0;
+
     /* FIXME: the LongData buffers must also be returned to the app */
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_open(WORD dev_id, MIDIOPENDESC *midi_desc, DWORD flags, struct notify_context *notify)
+static UINT midi_in_open(WORD dev_id, MIDIOPENDESC *midi_desc, UINT flags, struct notify_context *notify)
 {
     struct midi_src *src;
 
@@ -915,7 +938,7 @@ static DWORD midi_in_open(WORD dev_id, MIDIOPENDESC *midi_desc, DWORD flags, str
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_close(WORD dev_id, struct notify_context *notify)
+static UINT midi_in_close(WORD dev_id, struct notify_context *notify)
 {
     struct midi_src *src;
 
@@ -941,7 +964,7 @@ static DWORD midi_in_close(WORD dev_id, struct notify_context *notify)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_add_buffer(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
+static UINT midi_in_add_buffer(WORD dev_id, MIDIHDR *hdr, UINT hdr_size)
 {
     MIDIHDR **next;
 
@@ -984,7 +1007,7 @@ static DWORD midi_in_add_buffer(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_prepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
+static UINT midi_in_prepare(WORD dev_id, MIDIHDR *hdr, UINT hdr_size)
 {
     TRACE("dev_id = %d hdr = %p hdr_size = %d\n", dev_id, hdr, hdr_size);
 
@@ -999,7 +1022,7 @@ static DWORD midi_in_prepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_unprepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
+static UINT midi_in_unprepare(WORD dev_id, MIDIHDR *hdr, UINT hdr_size)
 {
     TRACE("dev_id = %d hdr = %p hdr_size = %d\n", dev_id, hdr, hdr_size);
 
@@ -1014,7 +1037,7 @@ static DWORD midi_in_unprepare(WORD dev_id, MIDIHDR *hdr, DWORD hdr_size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_get_devcaps(WORD dev_id, MIDIINCAPSW *caps, DWORD size)
+static UINT midi_in_get_devcaps(WORD dev_id, MIDIINCAPSW *caps, UINT size)
 {
     TRACE("dev_id = %d caps = %p size = %d\n", dev_id, caps, size);
 
@@ -1033,13 +1056,13 @@ static DWORD midi_in_get_devcaps(WORD dev_id, MIDIINCAPSW *caps, DWORD size)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_get_num_devs(void)
+static UINT midi_in_get_num_devs(void)
 {
     TRACE("\n");
     return num_srcs;
 }
 
-static DWORD midi_in_start(WORD dev_id)
+static UINT midi_in_start(WORD dev_id)
 {
     TRACE("%d\n", dev_id);
 
@@ -1053,7 +1076,7 @@ static DWORD midi_in_start(WORD dev_id)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_stop(WORD dev_id)
+static UINT midi_in_stop(WORD dev_id)
 {
     TRACE("%d\n", dev_id);
 
@@ -1066,10 +1089,10 @@ static DWORD midi_in_stop(WORD dev_id)
     return MMSYSERR_NOERROR;
 }
 
-static DWORD midi_in_reset(WORD dev_id, struct notify_context *notify)
+static UINT midi_in_reset(WORD dev_id, struct notify_context *notify)
 {
-    DWORD cur_time = get_time_ms();
-    DWORD err = MMSYSERR_NOERROR;
+    UINT cur_time = get_time_ms();
+    UINT err = MMSYSERR_NOERROR;
     struct midi_src *src;
     MIDIHDR *hdr;
 
@@ -1090,7 +1113,7 @@ static DWORD midi_in_reset(WORD dev_id, struct notify_context *notify)
         src->lpQueueHdr = hdr->lpNext;
         hdr->dwFlags &= ~MHDR_INQUEUE;
         hdr->dwFlags |= MHDR_DONE;
-        set_in_notify(notify, src, dev_id, MIM_LONGDATA, (DWORD_PTR)hdr, cur_time - src->startTime);
+        set_in_notify(notify, src, dev_id, MIM_LONGDATA, (UINT_PTR)hdr, cur_time - src->startTime);
         if (src->lpQueueHdr) err = ERROR_RETRY; /* ask the client to call again */
     }
 
@@ -1138,7 +1161,7 @@ NTSTATUS midi_out_message(void *args)
         *params->err = midi_out_get_num_devs();
         break;
     case MODM_GETVOLUME:
-        *params->err = midi_out_get_volume(params->dev_id, (DWORD *)params->param_1);
+        *params->err = midi_out_get_volume(params->dev_id, (UINT *)params->param_1);
         break;
     case MODM_SETVOLUME:
         *params->err = midi_out_set_volume(params->dev_id, params->param_1);
@@ -1222,3 +1245,370 @@ NTSTATUS midi_notify_wait(void *args)
 
     return STATUS_SUCCESS;
 }
+
+#ifdef _WIN64
+
+typedef UINT PTR32;
+
+NTSTATUS wow64_midi_init(void *args)
+{
+    struct
+    {
+        PTR32 err;
+    } *params32 = args;
+    struct midi_init_params params =
+    {
+        .err = ULongToPtr(params32->err)
+    };
+    return midi_init(&params);
+}
+
+struct notify_context32
+{
+    BOOL send_notify;
+    WORD dev_id;
+    WORD msg;
+    UINT param_1;
+    UINT param_2;
+    UINT callback;
+    UINT flags;
+    PTR32 device;
+    UINT instance;
+};
+
+static void notify_to_notify32(struct notify_context32 *notify32,
+                               const struct notify_context *notify)
+{
+    notify32->send_notify = notify->send_notify;
+    notify32->dev_id = notify->dev_id;
+    notify32->msg = notify->msg;
+    notify32->param_1 = notify->param_1;
+    notify32->param_2 = notify->param_2;
+    notify32->callback = notify->callback;
+    notify32->flags = notify->flags;
+    notify32->device = PtrToUlong(notify->device);
+    notify32->instance = notify->instance;
+}
+
+struct midi_open_desc32
+{
+    PTR32 hMidi;
+    UINT dwCallback;
+    UINT dwInstance;
+    UINT dnDevNode;
+    UINT cIds;
+    MIDIOPENSTRMID rgIds;
+};
+
+struct midi_hdr32
+{
+    PTR32 lpData;
+    UINT dwBufferLength;
+    UINT dwBytesRecorded;
+    UINT dwUser;
+    UINT dwFlags;
+    PTR32 lpNext;
+    UINT reserved;
+    UINT dwOffset;
+    UINT dwReserved[8];
+};
+
+static UINT wow64_midi_out_prepare(WORD dev_id, struct midi_hdr32 *hdr, UINT hdr_size)
+{
+    TRACE("(%04X, %p, %d);\n", dev_id, hdr, hdr_size);
+
+    if (hdr_size < offsetof(struct midi_hdr32, dwOffset) || !hdr || !hdr->lpData)
+        return MMSYSERR_INVALPARAM;
+    if (hdr->dwFlags & MHDR_PREPARED)
+        return MMSYSERR_NOERROR;
+
+    hdr->lpNext = 0;
+    hdr->dwFlags |= MHDR_PREPARED;
+    hdr->dwFlags &= ~(MHDR_DONE | MHDR_INQUEUE);
+    return MMSYSERR_NOERROR;
+}
+
+static UINT wow64_midi_out_unprepare(WORD dev_id, struct midi_hdr32 *hdr, UINT hdr_size)
+{
+    TRACE("(%04X, %p, %d);\n", dev_id, hdr, hdr_size);
+
+    if (hdr_size < offsetof(struct midi_hdr32, dwOffset) || !hdr || !hdr->lpData)
+        return MMSYSERR_INVALPARAM;
+    if (!(hdr->dwFlags & MHDR_PREPARED))
+        return MMSYSERR_NOERROR;
+    if (hdr->dwFlags & MHDR_INQUEUE)
+        return MIDIERR_STILLPLAYING;
+
+    hdr->dwFlags &= ~MHDR_PREPARED;
+    return MMSYSERR_NOERROR;
+}
+
+NTSTATUS wow64_midi_out_message(void *args)
+{
+    struct
+    {
+        UINT dev_id;
+        UINT msg;
+        UINT user;
+        UINT param_1;
+        UINT param_2;
+        PTR32 err;
+        PTR32 notify;
+    } *params32 = args;
+    struct notify_context32 *notify32 = ULongToPtr(params32->notify);
+    struct midi_open_desc32 *desc32;
+    struct midi_hdr32 *hdr32;
+    struct notify_context notify;
+    MIDIOPENDESC open_desc;
+    MIDIHDR hdr;
+    struct midi_out_message_params params =
+    {
+        .dev_id = params32->dev_id,
+        .msg = params32->msg,
+        .user = params32->user,
+        .param_1 = params32->param_1,
+        .param_2 = params32->param_2,
+        .err = ULongToPtr(params32->err),
+        .notify = &notify
+    };
+    notify32->send_notify = FALSE;
+
+    switch (params32->msg)
+    {
+    case MODM_OPEN:
+        desc32 = ULongToPtr(params32->param_1);
+
+        open_desc.hMidi = ULongToPtr(desc32->hMidi);
+        open_desc.dwCallback = desc32->dwCallback;
+        open_desc.dwInstance = desc32->dwInstance;
+        open_desc.dnDevNode = desc32->dnDevNode;
+        open_desc.cIds = desc32->cIds;
+        open_desc.rgIds.dwStreamID = desc32->rgIds.dwStreamID;
+        open_desc.rgIds.wDeviceID = desc32->rgIds.wDeviceID;
+
+        params.param_1 = (UINT_PTR)&open_desc;
+        break;
+
+    case MODM_LONGDATA:
+        hdr32 = ULongToPtr(params32->param_1);
+
+        memset(&hdr, 0, sizeof(hdr));
+        hdr.lpData = ULongToPtr(hdr32->lpData);
+        hdr.dwBufferLength = hdr32->dwBufferLength;
+        hdr.dwFlags = hdr32->dwFlags;
+
+        params.param_1 = (UINT_PTR)&hdr;
+        params.param_2 = sizeof(hdr);
+        break;
+
+    case MODM_PREPARE: /* prepare and unprepare are easier to handle explicitly */
+        hdr32 = ULongToPtr(params32->param_1);
+
+        *params.err = wow64_midi_out_prepare(params32->dev_id, hdr32, params32->param_2);
+        return STATUS_SUCCESS;
+
+    case MODM_UNPREPARE:
+        hdr32 = ULongToPtr(params32->param_1);
+
+        *params.err = wow64_midi_out_unprepare(params32->dev_id, hdr32, params32->param_2);
+        return STATUS_SUCCESS;
+    }
+
+    midi_out_message(&params);
+
+    switch (params32->msg)
+    {
+    case MODM_LONGDATA:
+        hdr32 = ULongToPtr(params32->param_1);
+
+        hdr32->dwFlags = hdr.dwFlags;
+        break;
+    }
+
+    if (notify.send_notify)
+    {
+        notify_to_notify32(notify32, &notify);
+
+        if (notify.msg == MOM_DONE)
+            notify32->param_1 = params32->param_1; /* restore the 32-bit hdr */
+    }
+    return STATUS_SUCCESS;
+}
+
+static UINT wow64_midi_in_prepare(WORD dev_id, struct midi_hdr32 *hdr, UINT hdr_size)
+{
+    TRACE("(%04X, %p, %d);\n", dev_id, hdr, hdr_size);
+
+    if (hdr_size < offsetof(struct midi_hdr32, dwOffset) || !hdr || !hdr->lpData)
+        return MMSYSERR_INVALPARAM;
+    if (hdr->dwFlags & MHDR_PREPARED)
+        return MMSYSERR_NOERROR;
+
+    hdr->lpNext = 0;
+    hdr->dwFlags |= MHDR_PREPARED;
+    hdr->dwFlags &= ~(MHDR_DONE | MHDR_INQUEUE);
+
+    return MMSYSERR_NOERROR;
+}
+
+static UINT wow64_midi_in_unprepare(WORD dev_id, struct midi_hdr32 *hdr, UINT hdr_size)
+{
+    TRACE("(%04X, %p, %d);\n", dev_id, hdr, hdr_size);
+
+    if (hdr_size < offsetof(struct midi_hdr32, dwOffset) || !hdr || !hdr->lpData)
+        return MMSYSERR_INVALPARAM;
+    if (!(hdr->dwFlags & MHDR_PREPARED))
+        return MMSYSERR_NOERROR;
+    if (hdr->dwFlags & MHDR_INQUEUE)
+        return MIDIERR_STILLPLAYING;
+
+    hdr->dwFlags &= ~MHDR_PREPARED;
+
+    return MMSYSERR_NOERROR;
+}
+
+NTSTATUS wow64_midi_in_message(void *args)
+{
+    struct
+    {
+        UINT dev_id;
+        UINT msg;
+        UINT user;
+        UINT param_1;
+        UINT param_2;
+        PTR32 err;
+        PTR32 notify;
+    } *params32 = args;
+    struct notify_context32 *notify32 = ULongToPtr(params32->notify);
+    struct midi_open_desc32 *desc32;
+    struct midi_hdr32 *hdr32;
+    struct notify_context notify;
+    MIDIOPENDESC open_desc;
+    MIDIHDR *hdr = NULL;
+    struct midi_in_message_params params =
+    {
+        .dev_id = params32->dev_id,
+        .msg = params32->msg,
+        .user = params32->user,
+        .param_1 = params32->param_1,
+        .param_2 = params32->param_2,
+        .err = ULongToPtr(params32->err),
+        .notify = &notify
+    };
+    notify32->send_notify = FALSE;
+
+    switch (params32->msg)
+    {
+    case MIDM_OPEN:
+        desc32 = ULongToPtr(params32->param_1);
+
+        open_desc.hMidi = ULongToPtr(desc32->hMidi);
+        open_desc.dwCallback = desc32->dwCallback;
+        open_desc.dwInstance = desc32->dwInstance;
+        open_desc.dnDevNode = desc32->dnDevNode;
+        open_desc.cIds = desc32->cIds;
+        open_desc.rgIds.dwStreamID = desc32->rgIds.dwStreamID;
+        open_desc.rgIds.wDeviceID = desc32->rgIds.wDeviceID;
+
+        params.param_1 = (UINT_PTR)&open_desc;
+        break;
+
+    case MIDM_ADDBUFFER:
+        hdr32 = ULongToPtr(params32->param_1);
+
+        hdr = calloc(1, sizeof(*hdr));
+        hdr->lpData = ULongToPtr(hdr32->lpData);
+        hdr->dwBufferLength = hdr32->dwBufferLength;
+        hdr->dwFlags = hdr32->dwFlags;
+        hdr->dwReserved[7] = params32->param_1; /* keep hdr32 for MIM_LONGDATA notification */
+
+        params.param_1 = (UINT_PTR)hdr;
+        params.param_2 = sizeof(*hdr);
+        break;
+
+    case MIDM_PREPARE: /* prepare and unprepare are easier to handle explicitly */
+        hdr32 = ULongToPtr(params32->param_1);
+
+        *params.err = wow64_midi_in_prepare(params32->dev_id, hdr32, params32->param_2);
+        return STATUS_SUCCESS;
+
+    case MIDM_UNPREPARE:
+        hdr32 = ULongToPtr(params32->param_1);
+
+        *params.err = wow64_midi_in_unprepare(params32->dev_id, hdr32, params32->param_2);
+        return STATUS_SUCCESS;
+    }
+
+    midi_in_message(&params);
+
+    switch (params32->msg)
+    {
+    case MIDM_ADDBUFFER:
+        hdr32 = ULongToPtr(params32->param_1);
+
+        if (!*params.err)
+        {
+            hdr32->dwFlags = hdr->dwFlags;
+            hdr32->dwBytesRecorded = hdr->dwBytesRecorded;
+            hdr32->lpNext = 0;
+        }
+        else
+            free(hdr);
+        break;
+    }
+
+    if (notify.send_notify)
+    {
+        notify_to_notify32(notify32, &notify);
+
+        if (notify.msg == MIM_LONGDATA)
+        {
+            hdr = (MIDIHDR *)notify.param_1;
+            notify32->param_1 = hdr->dwReserved[7];
+            hdr32 = ULongToPtr(notify32->param_1);
+            hdr32->dwBytesRecorded = hdr->dwBytesRecorded;
+            hdr32->dwFlags = hdr->dwFlags;
+            free(hdr);
+        }
+    }
+    return STATUS_SUCCESS;
+}
+
+NTSTATUS wow64_midi_notify_wait(void *args)
+{
+    struct
+    {
+        PTR32 quit;
+        PTR32 notify;
+    } *params32 = args;
+    struct notify_context32 *notify32 = ULongToPtr(params32->notify);
+    struct midi_hdr32 *hdr32;
+    struct notify_context notify;
+    MIDIHDR *hdr;
+    struct midi_notify_wait_params params =
+    {
+        .quit = ULongToPtr(params32->quit),
+        .notify = &notify
+    };
+    notify32->send_notify = FALSE;
+
+    midi_notify_wait(&params);
+
+    if (!*params.quit && notify.send_notify)
+    {
+        notify_to_notify32(notify32, &notify);
+
+        if (notify.msg == MIM_LONGDATA)
+        {
+            hdr = (MIDIHDR *)notify.param_1;
+            notify32->param_1 = hdr->dwReserved[7];
+            hdr32 = ULongToPtr(notify32->param_1);
+            hdr32->dwBytesRecorded = hdr->dwBytesRecorded;
+            hdr32->dwFlags = hdr->dwFlags;
+            free(hdr);
+        }
+    }
+    return STATUS_SUCCESS;
+}
+
+#endif /* _WIN64 */
