@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include "wined3d_private.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(d3d_shader);
@@ -307,6 +305,7 @@ enum wined3d_sm4_opcode
     WINED3D_SM5_OP_IMM_ATOMIC_UMIN                  = 0xbd,
     WINED3D_SM5_OP_SYNC                             = 0xbe,
     WINED3D_SM5_OP_EVAL_SAMPLE_INDEX                = 0xcc,
+    WINED3D_SM5_OP_EVAL_CENTROID                    = 0xcd,
     WINED3D_SM5_OP_DCL_GS_INSTANCES                 = 0xce,
 };
 
@@ -1126,6 +1125,7 @@ static const struct wined3d_sm4_opcode_info opcode_table[] =
     {WINED3D_SM5_OP_SYNC,                             WINED3DSIH_SYNC,                             "",     "",
             shader_sm5_read_sync},
     {WINED3D_SM5_OP_EVAL_SAMPLE_INDEX,                WINED3DSIH_EVAL_SAMPLE_INDEX,                "f",    "fi"},
+    {WINED3D_SM5_OP_EVAL_CENTROID,                    WINED3DSIH_EVAL_CENTROID,                    "f",    "f"},
     {WINED3D_SM5_OP_DCL_GS_INSTANCES,                 WINED3DSIH_DCL_GS_INSTANCES,                 "",     "",
             shader_sm4_read_declaration_count},
 };
@@ -1321,7 +1321,7 @@ static void *shader_sm4_init(const DWORD *byte_code, size_t byte_code_size,
         struct wined3d_shader_signature_element *e = &output_signature->elements[i];
 
         if (priv->shader_version.type == WINED3D_SHADER_TYPE_PIXEL
-                && _strnicmp(e->semantic_name, "SV_TARGET", -1))
+                && stricmp(e->semantic_name, "SV_TARGET"))
             continue;
         if (e->register_idx >= ARRAY_SIZE(priv->output_map))
         {
