@@ -70,13 +70,20 @@ struct wg_parser *wg_parser_create(enum wg_parser_type type, bool unlimited_buff
         .unlimited_buffering = unlimited_buffering,
     };
 
+    TRACE("type %#x, unlimited_buffering %d.\n", type, unlimited_buffering);
+
     if (__wine_unix_call(unix_handle, unix_wg_parser_create, &params))
         return NULL;
+
+    TRACE("Returning parser %p.\n", params.parser);
+
     return params.parser;
 }
 
 void wg_parser_destroy(struct wg_parser *parser)
 {
+    TRACE("parser %p.\n", parser);
+
     __wine_unix_call(unix_handle, unix_wg_parser_destroy, parser);
 }
 
@@ -88,11 +95,15 @@ HRESULT wg_parser_connect(struct wg_parser *parser, uint64_t file_size)
         .file_size = file_size,
     };
 
+    TRACE("parser %p, file_size %I64u.\n", parser, file_size);
+
     return __wine_unix_call(unix_handle, unix_wg_parser_connect, &params);
 }
 
 void wg_parser_disconnect(struct wg_parser *parser)
 {
+    TRACE("parser %p.\n", parser);
+
     __wine_unix_call(unix_handle, unix_wg_parser_disconnect, parser);
 }
 
@@ -103,6 +114,8 @@ bool wg_parser_get_next_read_offset(struct wg_parser *parser, uint64_t *offset,
         .parser = parser,
     };
 
+    TRACE("parser %p, offset %p, size %p.\n", parser, offset, size);
+
     if (__wine_unix_call(unix_handle, unix_wg_parser_get_next_read_offset, &params))
         return false;
     *offset = params.offset;
@@ -119,6 +132,8 @@ void wg_parser_push_data(struct wg_parser *parser, const void *data, uint32_t si
         .size = size,
     };
 
+    TRACE("parser %p, data %p, size %u.\n", parser, data, size);
+
     __wine_unix_call(unix_handle, unix_wg_parser_push_data, &params);
 }
 
@@ -129,6 +144,8 @@ uint32_t wg_parser_get_stream_count(struct wg_parser *parser)
         .parser = parser,
     };
 
+    TRACE("parser %p.\n", parser);
+
     __wine_unix_call(unix_handle, unix_wg_parser_get_stream_count, &params);
     return params.count;
 }
@@ -141,7 +158,11 @@ struct wg_parser_stream *wg_parser_get_stream(struct wg_parser *parser, uint32_t
         .index = index,
     };
 
+    TRACE("parser %p, index %u.\n", parser, index);
+
     __wine_unix_call(unix_handle, unix_wg_parser_get_stream, &params);
+
+    TRACE("Returning stream %p.\n", params.stream);
     return params.stream;
 }
 
@@ -153,6 +174,8 @@ void wg_parser_stream_get_preferred_format(struct wg_parser_stream *stream, stru
         .format = format,
     };
 
+    TRACE("stream %p, format %p.\n", stream, format);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_get_preferred_format, &params);
 }
 
@@ -164,11 +187,15 @@ void wg_parser_stream_enable(struct wg_parser_stream *stream, const struct wg_fo
         .format = format,
     };
 
+    TRACE("stream %p, format %p.\n", stream, format);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_enable, &params);
 }
 
 void wg_parser_stream_disable(struct wg_parser_stream *stream)
 {
+    TRACE("stream %p.\n", stream);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_disable, stream);
 }
 
@@ -180,6 +207,8 @@ bool wg_parser_stream_get_buffer(struct wg_parser_stream *stream, struct wg_pars
         .buffer = buffer,
     };
 
+    TRACE("stream %p, buffer %p.\n", stream, buffer);
+
     return !__wine_unix_call(unix_handle, unix_wg_parser_stream_get_buffer, &params);
 }
 
@@ -194,11 +223,15 @@ bool wg_parser_stream_copy_buffer(struct wg_parser_stream *stream,
         .size = size,
     };
 
+    TRACE("stream %p, data %p, offset %u, size %u.\n", stream, data, offset, size);
+
     return !__wine_unix_call(unix_handle, unix_wg_parser_stream_copy_buffer, &params);
 }
 
 void wg_parser_stream_release_buffer(struct wg_parser_stream *stream)
 {
+    TRACE("stream %p.\n", stream);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_release_buffer, stream);
 }
 
@@ -214,6 +247,9 @@ void wg_parser_stream_notify_qos(struct wg_parser_stream *stream,
         .timestamp = timestamp,
     };
 
+    TRACE("stream %p, underflow %d, proportion %.16e, diff %I64d, timestamp %I64u.\n",
+            stream, underflow, proportion, diff, timestamp);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_notify_qos, &params);
 }
 
@@ -224,7 +260,11 @@ uint64_t wg_parser_stream_get_duration(struct wg_parser_stream *stream)
         .stream = stream,
     };
 
+    TRACE("stream %p.\n", stream);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_get_duration, &params);
+
+    TRACE("Returning duration %I64u.\n", params.duration);
     return params.duration;
 }
 
@@ -241,6 +281,9 @@ void wg_parser_stream_seek(struct wg_parser_stream *stream, double rate,
         .stop_flags = stop_flags,
     };
 
+    TRACE("stream %p, rate %.16e, start_pos %I64u, stop_pos %I64u, start_flags %#lx, stop_flags %#lx.\n",
+            stream, rate, start_pos, stop_pos, start_flags, stop_flags);
+
     __wine_unix_call(unix_handle, unix_wg_parser_stream_seek, &params);
 }
 
@@ -253,16 +296,58 @@ struct wg_transform *wg_transform_create(const struct wg_format *input_format,
         .output_format = output_format,
     };
 
+    TRACE("input_format %p, output_format %p.\n", input_format, output_format);
+
     if (__wine_unix_call(unix_handle, unix_wg_transform_create, &params))
         return NULL;
+
+    TRACE("Returning transform %p.\n", params.transform);
     return params.transform;
 }
 
 void wg_transform_destroy(struct wg_transform *transform)
 {
+    TRACE("transform %p.\n", transform);
+
     __wine_unix_call(unix_handle, unix_wg_transform_destroy, transform);
 }
 
+HRESULT wg_transform_push_data(struct wg_transform *transform, struct wg_sample *sample)
+{
+    struct wg_transform_push_data_params params =
+    {
+        .transform = transform,
+        .sample = sample,
+    };
+    NTSTATUS status;
+
+    TRACE("transform %p, sample %p.\n", transform, sample);
+
+    if ((status = __wine_unix_call(unix_handle, unix_wg_transform_push_data, &params)))
+        return HRESULT_FROM_NT(status);
+
+    return params.result;
+}
+
+HRESULT wg_transform_read_data(struct wg_transform *transform, struct wg_sample *sample,
+        struct wg_format *format)
+{
+    struct wg_transform_read_data_params params =
+    {
+        .transform = transform,
+        .sample = sample,
+        .format = format,
+    };
+    NTSTATUS status;
+
+    TRACE("transform %p, sample %p, format %p.\n", transform, sample, format);
+
+    if ((status = __wine_unix_call(unix_handle, unix_wg_transform_read_data, &params)))
+        return HRESULT_FROM_NT(status);
+
+    return params.result;
+}
+
 BOOL WINAPI DllMain(HINSTANCE instance, DWORD reason, void *reserved)
 {
     if (reason == DLL_PROCESS_ATTACH)
@@ -348,9 +433,12 @@ static const IClassFactoryVtbl class_factory_vtbl =
 
 static struct class_factory avi_splitter_cf = {{&class_factory_vtbl}, avi_splitter_create};
 static struct class_factory decodebin_parser_cf = {{&class_factory_vtbl}, decodebin_parser_create};
+static struct class_factory mpeg_audio_codec_cf = {{&class_factory_vtbl}, mpeg_audio_codec_create};
 static struct class_factory mpeg_splitter_cf = {{&class_factory_vtbl}, mpeg_splitter_create};
 static struct class_factory wave_parser_cf = {{&class_factory_vtbl}, wave_parser_create};
 static struct class_factory wma_decoder_cf = {{&class_factory_vtbl}, wma_decoder_create};
+static struct class_factory resampler_cf = {{&class_factory_vtbl}, resampler_create};
+static struct class_factory color_convert_cf = {{&class_factory_vtbl}, color_convert_create};
 
 HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, void **out)
 {
@@ -369,12 +457,18 @@ HRESULT WINAPI DllGetClassObject(REFCLSID clsid, REFIID iid, void **out)
         factory = &avi_splitter_cf;
     else if (IsEqualGUID(clsid, &CLSID_decodebin_parser))
         factory = &decodebin_parser_cf;
+    else if (IsEqualGUID(clsid, &CLSID_CMpegAudioCodec))
+        factory = &mpeg_audio_codec_cf;
     else if (IsEqualGUID(clsid, &CLSID_MPEG1Splitter))
         factory = &mpeg_splitter_cf;
     else if (IsEqualGUID(clsid, &CLSID_WAVEParser))
         factory = &wave_parser_cf;
     else if (IsEqualGUID(clsid, &CLSID_WMADecMediaObject))
         factory = &wma_decoder_cf;
+    else if (IsEqualGUID(clsid, &CLSID_CResamplerMediaObject))
+        factory = &resampler_cf;
+    else if (IsEqualGUID(clsid, &CLSID_CColorConvertDMO))
+        factory = &color_convert_cf;
     else
     {
         FIXME("%s not implemented, returning CLASS_E_CLASSNOTAVAILABLE.\n", debugstr_guid(clsid));
@@ -434,6 +528,39 @@ static const REGFILTER2 reg_avi_splitter =
     .u.s2.rgPins2 = reg_avi_splitter_pins,
 };
 
+static const REGPINTYPES reg_mpeg_audio_codec_sink_mts[3] =
+{
+    {&MEDIATYPE_Audio, &MEDIASUBTYPE_MPEG1Packet},
+    {&MEDIATYPE_Audio, &MEDIASUBTYPE_MPEG1Payload},
+    {&MEDIATYPE_Audio, &MEDIASUBTYPE_MPEG1AudioPayload},
+};
+
+static const REGPINTYPES reg_mpeg_audio_codec_source_mts[1] =
+{
+    {&MEDIATYPE_Audio, &MEDIASUBTYPE_PCM},
+};
+
+static const REGFILTERPINS2 reg_mpeg_audio_codec_pins[2] =
+{
+    {
+        .nMediaTypes = 3,
+        .lpMediaType = reg_mpeg_audio_codec_sink_mts,
+    },
+    {
+        .dwFlags = REG_PINFLAG_B_OUTPUT,
+        .nMediaTypes = 1,
+        .lpMediaType = reg_mpeg_audio_codec_source_mts,
+    },
+};
+
+static const REGFILTER2 reg_mpeg_audio_codec =
+{
+    .dwVersion = 2,
+    .dwMerit = 0x03680001,
+    .u.s2.cPins2 = 2,
+    .u.s2.rgPins2 = reg_mpeg_audio_codec_pins,
+};
+
 static const REGPINTYPES reg_mpeg_splitter_sink_mts[4] =
 {
     {&MEDIATYPE_Stream, &MEDIASUBTYPE_MPEG1Audio},
@@ -536,6 +663,11 @@ static const REGFILTER2 reg_decodebin_parser =
 
 HRESULT WINAPI DllRegisterServer(void)
 {
+    DMO_PARTIAL_MEDIATYPE audio_convert_types[2] =
+    {
+        {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_PCM},
+        {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_IEEE_FLOAT},
+    };
     DMO_PARTIAL_MEDIATYPE wma_decoder_output[2] =
     {
         {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_PCM},
@@ -548,6 +680,48 @@ HRESULT WINAPI DllRegisterServer(void)
         {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_WMAUDIO3},
         {.type = MEDIATYPE_Audio, .subtype = MEDIASUBTYPE_WMAUDIO_LOSSLESS},
     };
+    DMO_PARTIAL_MEDIATYPE color_convert_input[20] =
+    {
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YV12},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YUY2},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_UYVY},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_AYUV},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_NV12},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB32},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB565},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_I420},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_IYUV},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YVYU},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB24},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB555},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB8},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_V216},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_V410},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_NV11},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_Y41P},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_Y41T},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_Y42T},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YVU9},
+    };
+    DMO_PARTIAL_MEDIATYPE color_convert_output[16] =
+    {
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YV12},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YUY2},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_UYVY},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_AYUV},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_NV12},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB32},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB565},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_I420},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_IYUV},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_YVYU},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB24},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB555},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_RGB8},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_V216},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_V410},
+        {.type = MEDIATYPE_Video, .subtype = MEDIASUBTYPE_NV11},
+    };
 
     IFilterMapper2 *mapper;
     HRESULT hr;
@@ -564,6 +738,8 @@ HRESULT WINAPI DllRegisterServer(void)
     IFilterMapper2_RegisterFilter(mapper, &CLSID_AviSplitter, L"AVI Splitter", NULL, NULL, NULL, &reg_avi_splitter);
     IFilterMapper2_RegisterFilter(mapper, &CLSID_decodebin_parser,
             L"GStreamer splitter filter", NULL, NULL, NULL, &reg_decodebin_parser);
+    IFilterMapper2_RegisterFilter(mapper, &CLSID_CMpegAudioCodec,
+            L"MPEG Audio Decoder", NULL, NULL, NULL, &reg_mpeg_audio_codec);
     IFilterMapper2_RegisterFilter(mapper, &CLSID_MPEG1Splitter,
             L"MPEG-I Stream Splitter", NULL, NULL, NULL, &reg_mpeg_splitter);
     IFilterMapper2_RegisterFilter(mapper, &CLSID_WAVEParser, L"Wave Parser", NULL, NULL, NULL, &reg_wave_parser);
@@ -573,6 +749,12 @@ HRESULT WINAPI DllRegisterServer(void)
     if (FAILED(hr = DMORegister(L"WMA Decoder DMO", &CLSID_WMADecMediaObject, &DMOCATEGORY_AUDIO_DECODER,
             0, ARRAY_SIZE(wma_decoder_input), wma_decoder_input, ARRAY_SIZE(wma_decoder_output), wma_decoder_output)))
         return hr;
+    if (FAILED(hr = DMORegister(L"Resampler DMO", &CLSID_CResamplerMediaObject, &DMOCATEGORY_AUDIO_EFFECT,
+            0, ARRAY_SIZE(audio_convert_types), audio_convert_types, ARRAY_SIZE(audio_convert_types), audio_convert_types)))
+        return hr;
+    if (FAILED(hr = DMORegister(L"Color Converter DMO", &CLSID_CColorConvertDMO, &DMOCATEGORY_VIDEO_EFFECT,
+            0, ARRAY_SIZE(color_convert_input), color_convert_input, ARRAY_SIZE(color_convert_output), color_convert_output)))
+        return hr;
 
     return mfplat_DllRegisterServer();
 }
@@ -593,11 +775,16 @@ HRESULT WINAPI DllUnregisterServer(void)
 
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_AviSplitter);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_decodebin_parser);
+    IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_CMpegAudioCodec);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_MPEG1Splitter);
     IFilterMapper2_UnregisterFilter(mapper, NULL, NULL, &CLSID_WAVEParser);
 
     IFilterMapper2_Release(mapper);
 
+    if (FAILED(hr = DMOUnregister(&CLSID_CColorConvertDMO, &DMOCATEGORY_VIDEO_EFFECT)))
+        return hr;
+    if (FAILED(hr = DMOUnregister(&CLSID_CResamplerMediaObject, &DMOCATEGORY_AUDIO_EFFECT)))
+        return hr;
     if (FAILED(hr = DMOUnregister(&CLSID_WMADecMediaObject, &DMOCATEGORY_AUDIO_DECODER)))
         return hr;
 
