@@ -23,12 +23,20 @@
 #include "ksmedia.h"
 #include "wmcodecdsp.h"
 #include "initguid.h"
+#include "d3d9types.h"
 #include "mfapi.h"
 
 #include "wine/debug.h"
+#include "wine/list.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mfplat);
 
+DEFINE_GUID(DMOVideoFormat_RGB32,D3DFMT_X8R8G8B8,0x524f,0x11ce,0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70);
+DEFINE_GUID(DMOVideoFormat_RGB24,D3DFMT_R8G8B8,0x524f,0x11ce,0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70);
+DEFINE_GUID(DMOVideoFormat_RGB565,D3DFMT_R5G6B5,0x524f,0x11ce,0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70);
+DEFINE_GUID(DMOVideoFormat_RGB555,D3DFMT_X1R5G5B5,0x524f,0x11ce,0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70);
+DEFINE_GUID(DMOVideoFormat_RGB8,D3DFMT_P8,0x524f,0x11ce,0x9f,0x53,0x00,0x20,0xaf,0x0b,0xa7,0x70);
+
 struct video_processor
 {
     IMFTransform IMFTransform_iface;
@@ -396,8 +404,6 @@ failed:
 
 static const GUID CLSID_GStreamerByteStreamHandler = {0x317df618, 0x5e5a, 0x468a, {0x9f, 0x15, 0xd8, 0x27, 0xa9, 0xa0, 0x81, 0x62}};
 
-static const GUID CLSID_WINEAudioConverter = {0x6a170414,0xaad9,0x4693,{0xb8,0x06,0x3a,0x0c,0x47,0xc5,0x70,0xd6}};
-
 static const struct class_object
 {
     const GUID *clsid;
@@ -407,7 +413,7 @@ class_objects[] =
 {
     { &CLSID_VideoProcessorMFT, &video_processor_create },
     { &CLSID_GStreamerByteStreamHandler, &winegstreamer_stream_handler_create },
-    { &CLSID_WINEAudioConverter, &audio_converter_create },
+    { &CLSID_MSH264DecoderMFT, &h264_decoder_create },
 };
 
 HRESULT mfplat_get_class_object(REFCLSID rclsid, REFIID riid, void **obj)
@@ -436,88 +442,206 @@ HRESULT mfplat_get_class_object(REFCLSID rclsid, REFIID riid, void **obj)
     return CLASS_E_CLASSNOTAVAILABLE;
 }
 
-static WCHAR audio_converterW[] = L"Audio Converter";
-static const GUID *const audio_converter_supported_types[] =
-{
-    &MFAudioFormat_PCM,
-    &MFAudioFormat_Float,
-};
-
-static WCHAR wma_decoderW[] = L"WMAudio Decoder MFT";
-static const GUID *const wma_decoder_input_types[] =
-{
-    &MEDIASUBTYPE_MSAUDIO1,
-    &MFAudioFormat_WMAudioV8,
-    &MFAudioFormat_WMAudioV9,
-    &MFAudioFormat_WMAudio_Lossless,
-};
-static const GUID *const wma_decoder_output_types[] =
-{
-    &MFAudioFormat_PCM,
-    &MFAudioFormat_Float,
-};
-
-static const struct mft
-{
-    const GUID *clsid;
-    const GUID *category;
-    LPWSTR name;
-    const UINT32 flags;
-    const GUID *major_type;
-    const UINT32 input_types_count;
-    const GUID *const *input_types;
-    const UINT32 output_types_count;
-    const GUID *const *output_types;
-}
-mfts[] =
-{
-    {
-        &CLSID_WINEAudioConverter,
-        &MFT_CATEGORY_AUDIO_EFFECT,
-        audio_converterW,
-        MFT_ENUM_FLAG_SYNCMFT,
-        &MFMediaType_Audio,
-        ARRAY_SIZE(audio_converter_supported_types),
-        audio_converter_supported_types,
-        ARRAY_SIZE(audio_converter_supported_types),
-        audio_converter_supported_types,
-    },
-    {
-        &CLSID_WMADecMediaObject,
-        &MFT_CATEGORY_AUDIO_DECODER,
-        wma_decoderW,
-        MFT_ENUM_FLAG_SYNCMFT,
-        &MFMediaType_Audio,
-        ARRAY_SIZE(wma_decoder_input_types),
-        wma_decoder_input_types,
-        ARRAY_SIZE(wma_decoder_output_types),
-        wma_decoder_output_types,
-    },
-};
-
 HRESULT mfplat_DllRegisterServer(void)
 {
-    unsigned int i, j;
+    MFT_REGISTER_TYPE_INFO resampler_types[] =
+    {
+        {MFMediaType_Audio, MFAudioFormat_PCM},
+        {MFMediaType_Audio, MFAudioFormat_Float},
+    };
+
+    MFT_REGISTER_TYPE_INFO wma_decoder_input_types[] =
+    {
+        {MFMediaType_Audio, MEDIASUBTYPE_MSAUDIO1},
+        {MFMediaType_Audio, MFAudioFormat_WMAudioV8},
+        {MFMediaType_Audio, MFAudioFormat_WMAudioV9},
+        {MFMediaType_Audio, MFAudioFormat_WMAudio_Lossless},
+    };
+    MFT_REGISTER_TYPE_INFO wma_decoder_output_types[] =
+    {
+        {MFMediaType_Audio, MFAudioFormat_PCM},
+        {MFMediaType_Audio, MFAudioFormat_Float},
+    };
+
+    MFT_REGISTER_TYPE_INFO h264_decoder_input_types[] =
+    {
+        {MFMediaType_Video, MFVideoFormat_H264},
+        {MFMediaType_Video, MFVideoFormat_H264_ES},
+    };
+    MFT_REGISTER_TYPE_INFO h264_decoder_output_types[] =
+    {
+        {MFMediaType_Video, MFVideoFormat_NV12},
+        {MFMediaType_Video, MFVideoFormat_YV12},
+        {MFMediaType_Video, MFVideoFormat_IYUV},
+        {MFMediaType_Video, MFVideoFormat_I420},
+        {MFMediaType_Video, MFVideoFormat_YUY2},
+    };
+
+    MFT_REGISTER_TYPE_INFO video_processor_input_types[] =
+    {
+        {MFMediaType_Video, MFVideoFormat_IYUV},
+        {MFMediaType_Video, MFVideoFormat_YV12},
+        {MFMediaType_Video, MFVideoFormat_NV12},
+        {MFMediaType_Video, MFVideoFormat_YUY2},
+        {MFMediaType_Video, MFVideoFormat_ARGB32},
+        {MFMediaType_Video, MFVideoFormat_RGB32},
+        {MFMediaType_Video, MFVideoFormat_NV11},
+        {MFMediaType_Video, MFVideoFormat_AYUV},
+        {MFMediaType_Video, MFVideoFormat_UYVY},
+        {MFMediaType_Video, MEDIASUBTYPE_P208},
+        {MFMediaType_Video, MFVideoFormat_RGB24},
+        {MFMediaType_Video, MFVideoFormat_RGB555},
+        {MFMediaType_Video, MFVideoFormat_RGB565},
+        {MFMediaType_Video, MFVideoFormat_RGB8},
+        {MFMediaType_Video, MFVideoFormat_I420},
+        {MFMediaType_Video, MFVideoFormat_Y216},
+        {MFMediaType_Video, MFVideoFormat_v410},
+        {MFMediaType_Video, MFVideoFormat_Y41P},
+        {MFMediaType_Video, MFVideoFormat_Y41T},
+        {MFMediaType_Video, MFVideoFormat_Y42T},
+        {MFMediaType_Video, MFVideoFormat_YVYU},
+        {MFMediaType_Video, MFVideoFormat_420O},
+    };
+    MFT_REGISTER_TYPE_INFO video_processor_output_types[] =
+    {
+        {MFMediaType_Video, MFVideoFormat_IYUV},
+        {MFMediaType_Video, MFVideoFormat_YV12},
+        {MFMediaType_Video, MFVideoFormat_NV12},
+        {MFMediaType_Video, MFVideoFormat_YUY2},
+        {MFMediaType_Video, MFVideoFormat_ARGB32},
+        {MFMediaType_Video, MFVideoFormat_RGB32},
+        {MFMediaType_Video, MFVideoFormat_NV11},
+        {MFMediaType_Video, MFVideoFormat_AYUV},
+        {MFMediaType_Video, MFVideoFormat_UYVY},
+        {MFMediaType_Video, MEDIASUBTYPE_P208},
+        {MFMediaType_Video, MFVideoFormat_RGB24},
+        {MFMediaType_Video, MFVideoFormat_RGB555},
+        {MFMediaType_Video, MFVideoFormat_RGB565},
+        {MFMediaType_Video, MFVideoFormat_RGB8},
+        {MFMediaType_Video, MFVideoFormat_I420},
+        {MFMediaType_Video, MFVideoFormat_Y216},
+        {MFMediaType_Video, MFVideoFormat_v410},
+        {MFMediaType_Video, MFVideoFormat_Y41P},
+        {MFMediaType_Video, MFVideoFormat_Y41T},
+        {MFMediaType_Video, MFVideoFormat_Y42T},
+        {MFMediaType_Video, MFVideoFormat_YVYU},
+    };
+
+    MFT_REGISTER_TYPE_INFO color_convert_input_types[] =
+    {
+        {MFMediaType_Video, MFVideoFormat_YV12},
+        {MFMediaType_Video, MFVideoFormat_YUY2},
+        {MFMediaType_Video, MFVideoFormat_UYVY},
+        {MFMediaType_Video, MFVideoFormat_AYUV},
+        {MFMediaType_Video, MFVideoFormat_NV12},
+        {MFMediaType_Video, DMOVideoFormat_RGB32},
+        {MFMediaType_Video, DMOVideoFormat_RGB565},
+        {MFMediaType_Video, MFVideoFormat_I420},
+        {MFMediaType_Video, MFVideoFormat_IYUV},
+        {MFMediaType_Video, MFVideoFormat_YVYU},
+        {MFMediaType_Video, DMOVideoFormat_RGB24},
+        {MFMediaType_Video, DMOVideoFormat_RGB555},
+        {MFMediaType_Video, DMOVideoFormat_RGB8},
+        {MFMediaType_Video, MEDIASUBTYPE_V216},
+        {MFMediaType_Video, MEDIASUBTYPE_V410},
+        {MFMediaType_Video, MFVideoFormat_NV11},
+        {MFMediaType_Video, MFVideoFormat_Y41P},
+        {MFMediaType_Video, MFVideoFormat_Y41T},
+        {MFMediaType_Video, MFVideoFormat_Y42T},
+        {MFMediaType_Video, MFVideoFormat_YVU9},
+    };
+    MFT_REGISTER_TYPE_INFO color_convert_output_types[] =
+    {
+        {MFMediaType_Video, MFVideoFormat_YV12},
+        {MFMediaType_Video, MFVideoFormat_YUY2},
+        {MFMediaType_Video, MFVideoFormat_UYVY},
+        {MFMediaType_Video, MFVideoFormat_AYUV},
+        {MFMediaType_Video, MFVideoFormat_NV12},
+        {MFMediaType_Video, DMOVideoFormat_RGB32},
+        {MFMediaType_Video, DMOVideoFormat_RGB565},
+        {MFMediaType_Video, MFVideoFormat_I420},
+        {MFMediaType_Video, MFVideoFormat_IYUV},
+        {MFMediaType_Video, MFVideoFormat_YVYU},
+        {MFMediaType_Video, DMOVideoFormat_RGB24},
+        {MFMediaType_Video, DMOVideoFormat_RGB555},
+        {MFMediaType_Video, DMOVideoFormat_RGB8},
+        {MFMediaType_Video, MEDIASUBTYPE_V216},
+        {MFMediaType_Video, MEDIASUBTYPE_V410},
+        {MFMediaType_Video, MFVideoFormat_NV11},
+    };
+
+    struct mft
+    {
+        GUID clsid;
+        GUID category;
+        WCHAR name[MAX_PATH];
+        UINT32 flags;
+        UINT32 input_types_count;
+        MFT_REGISTER_TYPE_INFO *input_types;
+        UINT32 output_types_count;
+        MFT_REGISTER_TYPE_INFO *output_types;
+    }
+    mfts[] =
+    {
+        {
+            CLSID_WMADecMediaObject,
+            MFT_CATEGORY_AUDIO_DECODER,
+            L"WMAudio Decoder MFT",
+            MFT_ENUM_FLAG_SYNCMFT,
+            ARRAY_SIZE(wma_decoder_input_types),
+            wma_decoder_input_types,
+            ARRAY_SIZE(wma_decoder_output_types),
+            wma_decoder_output_types,
+        },
+        {
+            CLSID_MSH264DecoderMFT,
+            MFT_CATEGORY_VIDEO_DECODER,
+            L"Microsoft H264 Video Decoder MFT",
+            MFT_ENUM_FLAG_SYNCMFT,
+            ARRAY_SIZE(h264_decoder_input_types),
+            h264_decoder_input_types,
+            ARRAY_SIZE(h264_decoder_output_types),
+            h264_decoder_output_types,
+        },
+        {
+            CLSID_VideoProcessorMFT,
+            MFT_CATEGORY_VIDEO_PROCESSOR,
+            L"Microsoft Video Processor MFT",
+            MFT_ENUM_FLAG_SYNCMFT,
+            ARRAY_SIZE(video_processor_input_types),
+            video_processor_input_types,
+            ARRAY_SIZE(video_processor_output_types),
+            video_processor_output_types,
+        },
+        {
+            CLSID_CResamplerMediaObject,
+            MFT_CATEGORY_AUDIO_EFFECT,
+            L"Resampler MFT",
+            MFT_ENUM_FLAG_SYNCMFT,
+            ARRAY_SIZE(resampler_types),
+            resampler_types,
+            ARRAY_SIZE(resampler_types),
+            resampler_types,
+        },
+        {
+            CLSID_CColorConvertDMO,
+            MFT_CATEGORY_VIDEO_EFFECT,
+            L"Color Converter MFT",
+            MFT_ENUM_FLAG_SYNCMFT,
+            ARRAY_SIZE(color_convert_input_types),
+            color_convert_input_types,
+            ARRAY_SIZE(color_convert_output_types),
+            color_convert_output_types,
+        },
+    };
+
+    unsigned int i;
     HRESULT hr;
-    MFT_REGISTER_TYPE_INFO input_types[4], output_types[2];
 
     for (i = 0; i < ARRAY_SIZE(mfts); i++)
     {
-        const struct mft *cur = &mfts[i];
-
-        for (j = 0; j < cur->input_types_count; j++)
-        {
-            input_types[j].guidMajorType = *(cur->major_type);
-            input_types[j].guidSubtype = *(cur->input_types[j]);
-        }
-        for (j = 0; j < cur->output_types_count; j++)
-        {
-            output_types[j].guidMajorType = *(cur->major_type);
-            output_types[j].guidSubtype = *(cur->output_types[j]);
-        }
-
-        hr = MFTRegister(*(cur->clsid), *(cur->category), cur->name, cur->flags, cur->input_types_count,
-                    input_types, cur->output_types_count, output_types, NULL);
+        hr = MFTRegister(mfts[i].clsid, mfts[i].category, mfts[i].name, mfts[i].flags, mfts[i].input_types_count,
+                    mfts[i].input_types, mfts[i].output_types_count, mfts[i].output_types, NULL);
 
         if (FAILED(hr))
         {
@@ -525,6 +649,7 @@ HRESULT mfplat_DllRegisterServer(void)
             return hr;
         }
     }
+
     return S_OK;
 }
 
@@ -621,6 +746,20 @@ static IMFMediaType *mf_media_type_from_wg_format_video(const struct wg_format *
             IMFMediaType_SetUINT32(type, &MF_MT_ALL_SAMPLES_INDEPENDENT, TRUE);
             IMFMediaType_SetUINT32(type, &MF_MT_VIDEO_ROTATION, MFVideoRotationFormat_0);
 
+            if (!IsRectEmpty(&format->u.video.padding))
+            {
+                MFVideoArea aperture =
+                {
+                    .OffsetX = {.value = format->u.video.padding.left},
+                    .OffsetY = {.value = format->u.video.padding.top},
+                    .Area.cx = format->u.video.width - format->u.video.padding.right - format->u.video.padding.left,
+                    .Area.cy = format->u.video.height - format->u.video.padding.bottom - format->u.video.padding.top,
+                };
+
+                IMFMediaType_SetBlob(type, &MF_MT_MINIMUM_DISPLAY_APERTURE,
+                        (BYTE *)&aperture, sizeof(aperture));
+            }
+
             return type;
         }
     }
@@ -632,11 +771,12 @@ IMFMediaType *mf_media_type_from_wg_format(const struct wg_format *format)
 {
     switch (format->major_type)
     {
-        case WG_MAJOR_TYPE_UNKNOWN:
-            return NULL;
-
+        case WG_MAJOR_TYPE_H264:
         case WG_MAJOR_TYPE_WMA:
-            FIXME("WMA format not implemented!\n");
+        case WG_MAJOR_TYPE_MPEG1_AUDIO:
+            FIXME("Format %u not implemented!\n", format->major_type);
+            /* fallthrough */
+        case WG_MAJOR_TYPE_UNKNOWN:
             return NULL;
 
         case WG_MAJOR_TYPE_AUDIO:
@@ -702,7 +842,9 @@ static void mf_media_type_to_wg_format_audio(IMFMediaType *type, const GUID *sub
 static void mf_media_type_to_wg_format_video(IMFMediaType *type, const GUID *subtype, struct wg_format *format)
 {
     UINT64 frame_rate, frame_size;
+    MFVideoArea aperture;
     unsigned int i;
+    UINT32 size;
 
     if (FAILED(IMFMediaType_GetUINT64(type, &MF_MT_FRAME_SIZE, &frame_size)))
     {
@@ -716,6 +858,15 @@ static void mf_media_type_to_wg_format_video(IMFMediaType *type, const GUID *sub
     format->u.video.fps_n = 1;
     format->u.video.fps_d = 1;
 
+    if (SUCCEEDED(IMFMediaType_GetBlob(type, &MF_MT_MINIMUM_DISPLAY_APERTURE, (BYTE *)&aperture,
+            sizeof(aperture), &size)) && size == sizeof(aperture))
+    {
+        format->u.video.padding.left = aperture.OffsetX.value;
+        format->u.video.padding.top = aperture.OffsetY.value;
+        format->u.video.padding.right = format->u.video.width - aperture.Area.cx - aperture.OffsetX.value;
+        format->u.video.padding.bottom = format->u.video.height - aperture.Area.cy - aperture.OffsetY.value;
+    }
+
     if (SUCCEEDED(IMFMediaType_GetUINT64(type, &MF_MT_FRAME_RATE, &frame_rate)) && (UINT32)frame_rate)
     {
         format->u.video.fps_n = (UINT32)(frame_rate >> 32);
@@ -795,6 +946,38 @@ static void mf_media_type_to_wg_format_wma(IMFMediaType *type, const GUID *subty
     memcpy(format->u.wma.codec_data, codec_data, codec_data_len);
 }
 
+static void mf_media_type_to_wg_format_h264(IMFMediaType *type, struct wg_format *format)
+{
+    UINT64 frame_rate, frame_size;
+    UINT32 profile, level;
+
+    memset(format, 0, sizeof(*format));
+    format->major_type = WG_MAJOR_TYPE_H264;
+
+    if (SUCCEEDED(IMFMediaType_GetUINT64(type, &MF_MT_FRAME_SIZE, &frame_size)))
+    {
+        format->u.h264.width = frame_size >> 32;
+        format->u.h264.height = (UINT32)frame_size;
+    }
+
+    if (SUCCEEDED(IMFMediaType_GetUINT64(type, &MF_MT_FRAME_RATE, &frame_rate)) && (UINT32)frame_rate)
+    {
+        format->u.h264.fps_n = frame_rate >> 32;
+        format->u.h264.fps_d = (UINT32)frame_rate;
+    }
+    else
+    {
+        format->u.h264.fps_n = 1;
+        format->u.h264.fps_d = 1;
+    }
+
+    if (SUCCEEDED(IMFMediaType_GetUINT32(type, &MF_MT_MPEG2_PROFILE, &profile)))
+        format->u.h264.profile = profile;
+
+    if (SUCCEEDED(IMFMediaType_GetUINT32(type, &MF_MT_MPEG2_LEVEL, &level)))
+        format->u.h264.level = level;
+}
+
 void mf_media_type_to_wg_format(IMFMediaType *type, struct wg_format *format)
 {
     GUID major_type, subtype;
@@ -823,7 +1006,12 @@ void mf_media_type_to_wg_format(IMFMediaType *type, struct wg_format *format)
             mf_media_type_to_wg_format_audio(type, &subtype, format);
     }
     else if (IsEqualGUID(&major_type, &MFMediaType_Video))
-        mf_media_type_to_wg_format_video(type, &subtype, format);
+    {
+        if (IsEqualGUID(&subtype, &MFVideoFormat_H264))
+            mf_media_type_to_wg_format_h264(type, format);
+        else
+            mf_media_type_to_wg_format_video(type, &subtype, format);
+    }
     else
         FIXME("Unrecognized major type %s.\n", debugstr_guid(&major_type));
 }
