@@ -35,6 +35,7 @@
 #include <gst/audio/audio.h>
 
 #include "winternl.h"
+#include "codecapi.h"
 #include "dshow.h"
 
 #include "unix_private.h"
@@ -176,7 +177,7 @@ static void wg_format_from_video_info(struct wg_format *format, const GstVideoIn
     format->u.video.fps_d = GST_VIDEO_INFO_FPS_D(info);
 }
 
-static void wg_format_from_caps_audio_mpeg(struct wg_format *format, const GstCaps *caps)
+static void wg_format_from_caps_audio_mpeg1(struct wg_format *format, const GstCaps *caps)
 {
     const GstStructure *structure = gst_caps_get_structure(caps, 0);
     gint layer, channels, rate;
@@ -197,17 +198,10 @@ static void wg_format_from_caps_audio_mpeg(struct wg_format *format, const GstCa
         return;
     }
 
-    format->major_type = WG_MAJOR_TYPE_AUDIO;
-
-    if (layer == 1)
-        format->u.audio.format = WG_AUDIO_FORMAT_MPEG1_LAYER1;
-    else if (layer == 2)
-        format->u.audio.format = WG_AUDIO_FORMAT_MPEG1_LAYER2;
-    else if (layer == 3)
-        format->u.audio.format = WG_AUDIO_FORMAT_MPEG1_LAYER3;
-
-    format->u.audio.channels = channels;
-    format->u.audio.rate = rate;
+    format->major_type = WG_MAJOR_TYPE_MPEG1_AUDIO;
+    format->u.mpeg1_audio.layer = layer;
+    format->u.mpeg1_audio.channels = channels;
+    format->u.mpeg1_audio.rate = rate;
 }
 
 static void wg_format_from_caps_video_cinepak(struct wg_format *format, const GstCaps *caps)
@@ -262,7 +256,7 @@ void wg_format_from_caps(struct wg_format *format, const GstCaps *caps)
     }
     else if (!strcmp(name, "audio/mpeg"))
     {
-        wg_format_from_caps_audio_mpeg(format, caps);
+        wg_format_from_caps_audio_mpeg1(format, caps);
     }
     else if (!strcmp(name, "video/x-cinepak"))
     {
@@ -337,6 +331,22 @@ static void wg_channel_mask_to_gst(GstAudioChannelPosition *positions, uint32_t
     }
 }
 
+static GstCaps *wg_format_to_caps_mpeg1_audio(const struct wg_format *format)
+{
+    GstCaps *caps;
+
+    if (!(caps = gst_caps_new_empty_simple("audio/mpeg")))
+        return NULL;
+
+    gst_caps_set_simple(caps, "mpegversion", G_TYPE_INT, 1, NULL);
+    gst_caps_set_simple(caps, "layer", G_TYPE_INT, format->u.mpeg1_audio.layer, NULL);
+    gst_caps_set_simple(caps, "rate", G_TYPE_INT, format->u.mpeg1_audio.rate, NULL);
+    gst_caps_set_simple(caps, "channels", G_TYPE_INT, format->u.mpeg1_audio.channels, NULL);
+    gst_caps_set_simple(caps, "parsed", G_TYPE_BOOLEAN, TRUE, NULL);
+
+    return caps;
+}
+
 static GstCaps *wg_format_to_caps_audio(const struct wg_format *format)
 {
     GstAudioChannelPosition positions[32];
@@ -384,11 +394,14 @@ static GstCaps *wg_format_to_caps_video(const struct wg_format *format)
     gst_video_info_set_format(&info, video_format, format->u.video.width, abs(format->u.video.height));
     if ((caps = gst_video_info_to_caps(&info)))
     {
-        /* Clear some fields that shouldn't prevent us from connecting. */
         for (i = 0; i < gst_caps_get_size(caps); ++i)
         {
-            gst_structure_remove_fields(gst_caps_get_structure(caps, i),
-                    "framerate", "pixel-aspect-ratio", "colorimetry", "chroma-site", NULL);
+            if (!format->u.video.width)
+                gst_structure_remove_fields(gst_caps_get_structure(caps, i), "width", NULL);
+            if (!format->u.video.height)
+                gst_structure_remove_fields(gst_caps_get_structure(caps, i), "height", NULL);
+            if (!format->u.video.fps_d && !format->u.video.fps_n)
+                gst_structure_remove_fields(gst_caps_get_structure(caps, i), "framerate", NULL);
         }
     }
     return caps;
@@ -431,14 +444,81 @@ static GstCaps *wg_format_to_caps_wma(const struct wg_format *format)
     return caps;
 }
 
+static GstCaps *wg_format_to_caps_h264(const struct wg_format *format)
+{
+    const char *profile, *level;
+    GstCaps *caps;
+
+    if (!(caps = gst_caps_new_empty_simple("video/x-h264")))
+        return NULL;
+    gst_caps_set_simple(caps, "stream-format", G_TYPE_STRING, "byte-stream", NULL);
+    gst_caps_set_simple(caps, "alignment", G_TYPE_STRING, "au", NULL);
+
+    if (format->u.h264.width)
+        gst_caps_set_simple(caps, "width", G_TYPE_INT, format->u.h264.width, NULL);
+    if (format->u.h264.height)
+        gst_caps_set_simple(caps, "height", G_TYPE_INT, format->u.h264.height, NULL);
+    if (format->u.h264.fps_n || format->u.h264.fps_d)
+        gst_caps_set_simple(caps, "framerate", GST_TYPE_FRACTION, format->u.h264.fps_n, format->u.h264.fps_d, NULL);
+
+    switch (format->u.h264.profile)
+    {
+        case eAVEncH264VProfile_Main: profile = "main"; break;
+        case eAVEncH264VProfile_High: profile = "high"; break;
+        case eAVEncH264VProfile_444:  profile = "high-4:4:4"; break;
+        default:
+            GST_FIXME("H264 profile attribute %u not implemented.", format->u.h264.profile);
+            /* fallthrough */
+        case eAVEncH264VProfile_unknown:
+            profile = NULL;
+            break;
+    }
+    if (profile)
+        gst_caps_set_simple(caps, "profile", G_TYPE_STRING, profile, NULL);
+
+    switch (format->u.h264.level)
+    {
+        case eAVEncH264VLevel1:   level = "1";   break;
+        case eAVEncH264VLevel1_1: level = "1.1"; break;
+        case eAVEncH264VLevel1_2: level = "1.2"; break;
+        case eAVEncH264VLevel1_3: level = "1.3"; break;
+        case eAVEncH264VLevel2:   level = "2";   break;
+        case eAVEncH264VLevel2_1: level = "2.1"; break;
+        case eAVEncH264VLevel2_2: level = "2.2"; break;
+        case eAVEncH264VLevel3:   level = "3";   break;
+        case eAVEncH264VLevel3_1: level = "3.1"; break;
+        case eAVEncH264VLevel3_2: level = "3.2"; break;
+        case eAVEncH264VLevel4:   level = "4";   break;
+        case eAVEncH264VLevel4_1: level = "4.1"; break;
+        case eAVEncH264VLevel4_2: level = "4.2"; break;
+        case eAVEncH264VLevel5:   level = "5";   break;
+        case eAVEncH264VLevel5_1: level = "5.1"; break;
+        case eAVEncH264VLevel5_2: level = "5.2"; break;
+        default:
+            GST_FIXME("H264 level attribute %u not implemented.", format->u.h264.level);
+            /* fallthrough */
+        case 0:
+            level = NULL;
+            break;
+    }
+    if (level)
+        gst_caps_set_simple(caps, "level", G_TYPE_STRING, level, NULL);
+
+    return caps;
+}
+
 GstCaps *wg_format_to_caps(const struct wg_format *format)
 {
     switch (format->major_type)
     {
         case WG_MAJOR_TYPE_UNKNOWN:
-            return NULL;
+            return gst_caps_new_any();
+        case WG_MAJOR_TYPE_MPEG1_AUDIO:
+            return wg_format_to_caps_mpeg1_audio(format);
         case WG_MAJOR_TYPE_WMA:
             return wg_format_to_caps_wma(format);
+        case WG_MAJOR_TYPE_H264:
+            return wg_format_to_caps_h264(format);
         case WG_MAJOR_TYPE_AUDIO:
             return wg_format_to_caps_audio(format);
         case WG_MAJOR_TYPE_VIDEO:
@@ -455,11 +535,12 @@ bool wg_format_compare(const struct wg_format *a, const struct wg_format *b)
 
     switch (a->major_type)
     {
-        case WG_MAJOR_TYPE_UNKNOWN:
-            return false;
-
+        case WG_MAJOR_TYPE_MPEG1_AUDIO:
         case WG_MAJOR_TYPE_WMA:
-            GST_FIXME("WMA format not implemented!\n");
+        case WG_MAJOR_TYPE_H264:
+            GST_FIXME("Format %u not implemented!", a->major_type);
+            /* fallthrough */
+        case WG_MAJOR_TYPE_UNKNOWN:
             return false;
 
         case WG_MAJOR_TYPE_AUDIO:
@@ -471,7 +552,8 @@ bool wg_format_compare(const struct wg_format *a, const struct wg_format *b)
             /* Do not compare FPS. */
             return a->u.video.format == b->u.video.format
                     && a->u.video.width == b->u.video.width
-                    && abs(a->u.video.height) == abs(b->u.video.height);
+                    && abs(a->u.video.height) == abs(b->u.video.height)
+                    && EqualRect( &a->u.video.padding, &b->u.video.padding );
     }
 
     assert(0);
